@@ -1,12 +1,12 @@
 #!/usr/bin/env python
 # vim:fileencoding=utf-8
 # License: GPLv3 Copyright: 2015, Kovid Goyal <kovid at kovidgoyal.net>
-from polyglot.builtins import map, unicode_type, environ_item, hasenv, getenv
+from polyglot.builtins import environ_item, hasenv
 import sys, locale, codecs, os, collections, collections.abc
 
 __appname__   = 'calibre'
-numeric_version = (5, 23, 0)
-__version__   = '.'.join(map(unicode_type, numeric_version))
+numeric_version = (5, 31, 1)
+__version__   = '.'.join(map(str, numeric_version))
 git_version   = None
 __author__    = "Kovid Goyal <kovid@kovidgoyal.net>"
 
@@ -118,18 +118,18 @@ def _get_cache_dir():
     confcache = os.path.join(config_dir, 'caches')
     try:
         os.makedirs(confcache)
-    except EnvironmentError as err:
+    except OSError as err:
         if err.errno != errno.EEXIST:
             raise
     if isportable:
         return confcache
-    ccd = getenv('CALIBRE_CACHE_DIRECTORY')
+    ccd = os.getenv('CALIBRE_CACHE_DIRECTORY')
     if ccd is not None:
         ans = os.path.abspath(ccd)
         try:
             os.makedirs(ans)
             return ans
-        except EnvironmentError as err:
+        except OSError as err:
             if err.errno == errno.EEXIST:
                 return ans
 
@@ -141,7 +141,7 @@ def _get_cache_dir():
     elif ismacos:
         candidate = os.path.join(os.path.expanduser('~/Library/Caches'), __appname__)
     else:
-        candidate = getenv('XDG_CACHE_HOME', '~/.cache')
+        candidate = os.getenv('XDG_CACHE_HOME', '~/.cache')
         candidate = os.path.join(os.path.expanduser(candidate),
                                     __appname__)
         if isinstance(candidate, bytes):
@@ -151,7 +151,7 @@ def _get_cache_dir():
                 candidate = confcache
     try:
         os.makedirs(candidate)
-    except EnvironmentError as err:
+    except OSError as err:
         if err.errno != errno.EEXIST:
             candidate = confcache
     return candidate
@@ -340,7 +340,7 @@ if plugins is None:
 
 CONFIG_DIR_MODE = 0o700
 
-cconfd = getenv('CALIBRE_CONFIG_DIRECTORY')
+cconfd = os.getenv('CALIBRE_CONFIG_DIRECTORY')
 if cconfd is not None:
     config_dir = os.path.abspath(cconfd)
 elif iswindows:
@@ -354,7 +354,7 @@ elif iswindows:
 elif ismacos:
     config_dir = os.path.expanduser('~/Library/Preferences/calibre')
 else:
-    bdir = os.path.abspath(os.path.expanduser(getenv('XDG_CONFIG_HOME', '~/.config')))
+    bdir = os.path.abspath(os.path.expanduser(os.getenv('XDG_CONFIG_HOME', '~/.config')))
     config_dir = os.path.join(bdir, 'calibre')
     try:
         os.makedirs(config_dir, mode=CONFIG_DIR_MODE)
@@ -363,7 +363,7 @@ else:
     if not os.path.exists(config_dir) or \
             not os.access(config_dir, os.W_OK) or not \
             os.access(config_dir, os.X_OK):
-        print('No write acces to', config_dir, 'using a temporary dir instead')
+        print('No write access to', config_dir, 'using a temporary dir instead')
         import tempfile, atexit
         config_dir = tempfile.mkdtemp(prefix='calibre-config-')
 
@@ -386,7 +386,7 @@ if getattr(sys, 'frozen', False):
     else:
         is_running_from_develop = running_in_develop_mode()
 
-in_develop_mode = getenv('CALIBRE_ENABLE_DEVELOP_MODE') == '1'
+in_develop_mode = os.getenv('CALIBRE_ENABLE_DEVELOP_MODE') == '1'
 
 
 def get_version():
@@ -415,7 +415,7 @@ def get_appname_for_display():
 def get_portable_base():
     'Return path to the directory that contains calibre-portable.exe or None'
     if isportable:
-        return os.path.dirname(os.path.dirname(getenv('CALIBRE_PORTABLE_BUILD')))
+        return os.path.dirname(os.path.dirname(os.getenv('CALIBRE_PORTABLE_BUILD')))
 
 
 def get_windows_username():
