@@ -15,7 +15,6 @@ from calibre.constants import islinux, ismacos
 from calibre.ptempfile import SpooledTemporaryFile
 from calibre.devices.errors import OpenFailed, DeviceError, BlacklistedDevice, OpenActionNeeded
 from calibre.devices.mtp.base import MTPDeviceBase, synchronous, debug
-from polyglot.builtins import unicode_type
 
 MTPDevice = namedtuple('MTPDevice', 'busnum devnum vendor_id product_id '
         'bcd serial manufacturer product')
@@ -76,7 +75,7 @@ class MTP_DEVICE(MTPDeviceBase):
                     traceback.print_stack()
                 return False
         if debug is not None and ans:
-            debug('Device {0} claims to be an MTP device in the IOKit registry'.format(d))
+            debug('Device {} claims to be an MTP device in the IOKit registry'.format(d))
         return bool(ans)
 
     def set_debug_level(self, lvl):
@@ -206,8 +205,7 @@ class MTP_DEVICE(MTPDeviceBase):
         self.dev = self._filesystem_cache = None
 
     def format_errorstack(self, errs):
-        return '\n'.join(['%d:%s'%(code, msg.decode('utf-8', 'replace')) for
-            code, msg in errs])
+        return '\n'.join('%d:%s'%(code, as_unicode(msg)) for code, msg in errs)
 
     @synchronous
     def open(self, connected_device, library_uuid):
@@ -223,7 +221,7 @@ class MTP_DEVICE(MTPDeviceBase):
         try:
             storage = sorted(self.dev.storage_info, key=operator.itemgetter('id'))
         except self.libmtp.MTPError as e:
-            if "The device has no storage information." in unicode_type(e):
+            if "The device has no storage information." in str(e):
                 # This happens on newer Android devices while waiting for
                 # the user to allow access. Apparently what happens is
                 # that when the user clicks allow, the device disconnects
@@ -318,7 +316,7 @@ class MTP_DEVICE(MTPDeviceBase):
                     storage.append({'id':sid, 'size':capacity,
                         'is_folder':True, 'name':name, 'can_delete':False,
                         'is_system':True})
-                    self._currently_getting_sid = unicode_type(sid)
+                    self._currently_getting_sid = str(sid)
                     items, errs = self.dev.get_filesystem(sid,
                             partial(self._filesystem_callback, {}))
                     all_items.extend(items), all_errs.extend(errs)
