@@ -1,4 +1,6 @@
 # -*- coding: utf-8 -*-
+
+
 __license__ = 'GPL 3'
 __copyright__ = '2009, Kovid Goyal <kovid@kovidgoyal.net>'
 __docformat__ = 'restructuredtext en'
@@ -18,25 +20,26 @@ from calibre.utils.zipfile import ZipFile
 from calibre import (extract, walk, isbytestring, filesystem_encoding,
         get_types_map)
 from calibre.constants import __version__
+from polyglot.builtins import string_or_bytes
 
-DEBUG_README=u'''
-This debug directory contains snapshots of the e-book as it passes through the
+DEBUG_README=b'''
+This debug folder contains snapshots of the e-book as it passes through the
 various stages of conversion. The stages are:
 
     1. input - This is the result of running the input plugin on the source
-    file. Use this directory to debug the input plugin.
+    file. Use this folder to debug the input plugin.
 
     2. parsed - This is the result of preprocessing and parsing the output of
     the input plugin. Note that for some input plugins this will be identical to
-    the input sub-directory. Use this directory to debug structure detection,
+    the input sub-folder. Use this folder to debug structure detection,
     etc.
 
     3. structure - This corresponds to the stage in the pipeline when structure
-    detection has run, but before the CSS is flattened. Use this directory to
+    detection has run, but before the CSS is flattened. Use this folder to
     debug the CSS flattening, font size conversion, etc.
 
     4. processed - This corresponds to the e-book as it is passed to the output
-    plugin. Use this directory to debug the output plugin.
+    plugin. Use this folder to debug the output plugin.
 
 '''
 
@@ -48,11 +51,11 @@ def supported_input_formats():
     return fmts
 
 
-class OptionValues(object):
+class OptionValues:
     pass
 
 
-class CompositeProgressReporter(object):
+class CompositeProgressReporter:
 
     def __init__(self, global_min, global_max, global_reporter):
         self.global_min, self.global_max = global_min, global_max
@@ -67,7 +70,7 @@ class CompositeProgressReporter(object):
 ARCHIVE_FMTS = ('zip', 'rar', 'oebzip')
 
 
-class Plumber(object):
+class Plumber:
 
     '''
     The `Plumber` manages the conversion pipeline. An UI should call the methods
@@ -86,7 +89,7 @@ class Plumber(object):
             override_input_metadata=False, for_regex_wizard=False, view_kepub=False):
         '''
         :param input: Path to input file.
-        :param output: Path to output file/directory
+        :param output: Path to output file/folder
         '''
         if isbytestring(input):
             input = input.decode(filesystem_encoding)
@@ -120,7 +123,7 @@ OptionRecommendation(name='debug_pipeline',
             short_switch='d',
             help=_('Save the output from different stages of the conversion '
                    'pipeline to the specified '
-                   'directory. Useful if you are unsure at which stage '
+                   'folder. Useful if you are unsure at which stage '
                    'of the conversion process a bug is occurring.')
         ),
 
@@ -131,7 +134,7 @@ OptionRecommendation(name='input_profile',
                    'conversion system information on how to interpret '
                    'various information in the input document. For '
                    'example resolution dependent lengths (i.e. lengths in '
-                   'pixels). Choices are:')+ ', '.join([
+                   'pixels). Choices are:') + ' ' + ', '.join([
                        x.short_name for x in input_profiles()])
         ),
 
@@ -151,7 +154,7 @@ OptionRecommendation(name='base_font_size',
             help=_('The base font size in pts. All font sizes in the produced book '
                    'will be rescaled based on this size. By choosing a larger '
                    'size you can make the fonts in the output bigger and vice '
-                   'versa. By default, the base font size is chosen based on '
+                   'versa. By default, when the value is zero, the base font size is chosen based on '
                    'the output profile you chose.'
                    )
         ),
@@ -263,8 +266,8 @@ OptionRecommendation(name='level2_toc',
 OptionRecommendation(name='level3_toc',
             recommended_value=None, level=OptionRecommendation.LOW,
             help=_('XPath expression that specifies all tags that should be '
-                'added to the Table of Contents at level three. Each entry '
-                'is added under the previous level two entry.'
+            'added to the Table of Contents at level three. Each entry '
+            'is added under the previous level two entry.'
             ' See the XPath Tutorial in the calibre User Manual for examples.'
                 )
         ),
@@ -294,8 +297,8 @@ OptionRecommendation(name='toc_threshold',
 OptionRecommendation(name='max_toc_links',
             recommended_value=50, level=OptionRecommendation.LOW,
             help=_('Maximum number of links to insert into the TOC. Set to 0 '
-               'to disable. Default is: %default. Links are only added to the '
-            'TOC if less than the threshold number of chapters were detected.'
+                'to disable. Default is: %default. Links are only added to the '
+                'TOC if less than the threshold number of chapters were detected.'
                 )
         ),
 
@@ -505,7 +508,7 @@ OptionRecommendation(name='smarten_punctuation',
         recommended_value=False, level=OptionRecommendation.LOW,
         help=_('Convert plain quotes, dashes and ellipsis to their '
             'typographically correct equivalents. For details, see '
-            'https://daringfireball.net/projects/smartypants'
+            'https://daringfireball.net/projects/smartypants.'
             )
         ),
 
@@ -526,17 +529,14 @@ OptionRecommendation(name='read_metadata_from_opf',
 
 OptionRecommendation(name='asciiize',
         recommended_value=False, level=OptionRecommendation.LOW,
-        help=(_('Transliterate unicode characters to an ASCII '
+        help=(_('Transliterate Unicode characters to an ASCII '
             'representation. Use with care because this will replace '
-            'unicode characters with ASCII. For instance it will replace "%s" '
-            'with "Mikhail Gorbachiov". Also, note that in '
+            'Unicode characters with ASCII. For instance it will replace "{0}" '
+            'with "{1}". Also, note that in '
             'cases where there are multiple representations of a character '
             '(characters shared by Chinese and Japanese for instance) the '
             'representation based on the current calibre interface language will be '
-            'used.')%
-            u'\u041c\u0438\u0445\u0430\u0438\u043b '
-            u'\u0413\u043e\u0440\u0431\u0430\u0447\u0451\u0432'
-)
+            'used.').format('Pelé', 'Pele'))
         ),
 
 OptionRecommendation(name='keep_ligatures',
@@ -794,7 +794,7 @@ OptionRecommendation(name='search_replace',
     def unarchive(self, path, tdir):
         extract(path, tdir)
         files = list(walk(tdir))
-        files = [f if isinstance(f, unicode) else f.decode(filesystem_encoding)
+        files = [f if isinstance(f, str) else f.decode(filesystem_encoding)
                 for f in files]
         from calibre.customize.ui import available_input_formats
         fmts = set(available_input_formats())
@@ -820,7 +820,7 @@ OptionRecommendation(name='search_replace',
         if not html_files:
             raise ValueError(_('Could not find an e-book inside the archive'))
         html_files = [(f, os.stat(f).st_size) for f in html_files]
-        html_files.sort(cmp=lambda x, y: cmp(x[1], y[1]))
+        html_files.sort(key=lambda x: x[1])
         html_files = [f[0] for f in html_files]
         for q in ('toc', 'index'):
             for f in html_files:
@@ -828,6 +828,14 @@ OptionRecommendation(name='search_replace',
                     return f, os.path.splitext(f)[1].lower()[1:]
         return html_files[-1], os.path.splitext(html_files[-1])[1].lower()[1:]
 
+    def get_all_options(self):
+        ans = {}
+        for group in (self.input_options, self.pipeline_options,
+                      self.output_options, self.all_format_options):
+            for rec in group:
+                ans[rec.option] = rec.recommended_value
+        return ans
+
     def get_option_by_name(self, name):
         for group in (self.input_options, self.pipeline_options,
                       self.output_options, self.all_format_options):
@@ -841,6 +849,16 @@ OptionRecommendation(name='search_replace',
         if help is not None:
             return help.replace('%default', str(rec.recommended_value))
 
+    def get_all_help(self):
+        ans = {}
+        for group in (self.input_options, self.pipeline_options,
+                      self.output_options, self.all_format_options):
+            for rec in group:
+                help = getattr(rec, 'help', None)
+                if help is not None:
+                    ans[rec.option.name] = help
+        return ans
+
     def merge_plugin_recs(self, plugin):
         for name, val, level in plugin.recommendations:
             rec = self.get_option_by_name(name)
@@ -897,19 +915,19 @@ OptionRecommendation(name='search_replace',
                     try:
                         val = parse_date(val, assume_utc=x=='timestamp')
                     except:
-                        self.log.exception(_('Failed to parse date/time') + ' ' + unicode(val))
+                        self.log.exception(_('Failed to parse date/time') + ' ' + str(val))
                         continue
                 setattr(mi, x, val)
 
     def download_cover(self, url):
         from calibre import browser
         from PIL import Image
-        from cStringIO import StringIO
+        import io
         from calibre.ptempfile import PersistentTemporaryFile
         self.log('Downloading cover from %r'%url)
         br = browser()
         raw = br.open_novisit(url).read()
-        buf = StringIO(raw)
+        buf = io.BytesIO(raw)
         pt = PersistentTemporaryFile('.jpg')
         pt.close()
         img = Image.open(buf)
@@ -927,8 +945,8 @@ OptionRecommendation(name='search_replace',
         if self.opts.read_metadata_from_opf is not None:
             self.opts.read_metadata_from_opf = os.path.abspath(
                                             self.opts.read_metadata_from_opf)
-            opf = OPF(open(self.opts.read_metadata_from_opf, 'rb'),
-                      os.path.dirname(self.opts.read_metadata_from_opf))
+            with lopen(self.opts.read_metadata_from_opf, 'rb') as stream:
+                opf = OPF(stream, os.path.dirname(self.opts.read_metadata_from_opf))
             mi = opf.to_book_metadata()
         self.opts_to_mi(mi)
         if mi.cover:
@@ -937,7 +955,8 @@ OptionRecommendation(name='search_replace',
             ext = mi.cover.rpartition('.')[-1].lower().strip()
             if ext not in ('png', 'jpg', 'jpeg', 'gif'):
                 ext = 'jpg'
-            mi.cover_data = (ext, open(mi.cover, 'rb').read())
+            with lopen(mi.cover, 'rb') as stream:
+                mi.cover_data = (ext, stream.read())
             mi.cover = None
         self.user_metadata = mi
 
@@ -993,7 +1012,7 @@ OptionRecommendation(name='search_replace',
         try:
             sys.stdout.flush()
             sys.stderr.flush()
-        except:
+        except Exception:
             pass
 
     def dump_oeb(self, oeb, out_dir):
@@ -1003,7 +1022,7 @@ OptionRecommendation(name='search_replace',
 
     def dump_input(self, ret, output_dir):
         out_dir = os.path.join(self.opts.debug_pipeline, 'input')
-        if isinstance(ret, basestring):
+        if isinstance(ret, string_or_bytes):
             shutil.copytree(output_dir, out_dir)
         else:
             if not os.path.exists(out_dir):
@@ -1033,17 +1052,17 @@ OptionRecommendation(name='search_replace',
         if self.opts.embed_all_fonts or self.opts.embed_font_family:
             # Start the threaded font scanner now, for performance
             from calibre.utils.fonts.scanner import font_scanner  # noqa
-        import cssutils, logging
-        cssutils.log.setLevel(logging.WARN)
-        get_types_map()  # Ensure the mimetypes module is intialized
+        import css_parser, logging
+        css_parser.log.setLevel(logging.WARN)
+        get_types_map()  # Ensure the mimetypes module is initialized
 
         if self.opts.debug_pipeline is not None:
             self.opts.verbose = max(self.opts.verbose, 4)
             self.opts.debug_pipeline = os.path.abspath(self.opts.debug_pipeline)
             if not os.path.exists(self.opts.debug_pipeline):
                 os.makedirs(self.opts.debug_pipeline)
-            open(os.path.join(self.opts.debug_pipeline, 'README.txt'),
-                    'w').write(DEBUG_README.encode('utf-8'))
+            with lopen(os.path.join(self.opts.debug_pipeline, 'README.txt'), 'wb') as f:
+                f.write(DEBUG_README)
             for x in ('input', 'parsed', 'structure', 'processed'):
                 x = os.path.join(self.opts.debug_pipeline, x)
                 if os.path.exists(x):
@@ -1060,7 +1079,7 @@ OptionRecommendation(name='search_replace',
 
         tdir = PersistentTemporaryDirectory('_plumber')
         stream = self.input if self.input_fmt == 'recipe' else \
-                open(self.input, 'rb')
+                lopen(self.input, 'rb')
         if self.input_fmt == 'recipe':
             self.opts.original_recipe_input_arg = self.original_input_arg
 
@@ -1081,6 +1100,7 @@ OptionRecommendation(name='search_replace',
         self.input_plugin.report_progress = ir
         if self.for_regex_wizard:
             self.input_plugin.for_viewer = True
+        self.output_plugin.specialize_options(self.log, self.opts, self.input_fmt)
         with self.input_plugin:
             self.oeb = self.input_plugin(stream, self.opts,
                                         self.input_fmt, self.log,
@@ -1152,8 +1172,8 @@ OptionRecommendation(name='search_replace',
             fkey = self.opts.dest.fkey
         else:
             try:
-                fkey = map(float, fkey.split(','))
-            except:
+                fkey = list(map(float, fkey.split(',')))
+            except Exception:
                 self.log.error('Invalid font size key: %r ignoring'%fkey)
                 fkey = self.opts.dest.fkey
 
@@ -1168,7 +1188,8 @@ OptionRecommendation(name='search_replace',
             self.log('Structured HTML written to:', out_dir)
 
         if self.opts.extra_css and os.path.exists(self.opts.extra_css):
-            self.opts.extra_css = open(self.opts.extra_css, 'rb').read()
+            with open(self.opts.extra_css, 'rb') as f:
+                self.opts.extra_css = f.read()
 
         oibl = self.opts.insert_blank_line
         orps  = self.opts.remove_paragraph_spacing
@@ -1194,7 +1215,7 @@ OptionRecommendation(name='search_replace',
         transform_css_rules = ()
         if self.opts.transform_css_rules:
             transform_css_rules = self.opts.transform_css_rules
-            if isinstance(transform_css_rules, basestring):
+            if isinstance(transform_css_rules, string_or_bytes):
                 transform_css_rules = json.loads(transform_css_rules)
         flattener = CSSFlattener(fbase=fbase, fkey=fkey,
                 lineh=line_height,
@@ -1291,3 +1312,16 @@ def create_oebbook(log, path_or_stream, opts, reader=None,
 
     reader()(oeb, path_or_stream)
     return oeb
+
+
+def create_dummy_plumber(input_format, output_format):
+    from calibre.utils.logging import Log
+    input_format = input_format.lower()
+    output_format = output_format.lower()
+    output_path = 'dummy.'+output_format
+    log = Log()
+    log.outputs = []
+    input_file = 'dummy.'+input_format
+    if input_format in ARCHIVE_FMTS:
+        input_file = 'dummy.html'
+    return Plumber(input_file, output_path, log)
