@@ -1,6 +1,6 @@
-#!/usr/bin/env python2
+#!/usr/bin/env python
 # coding: utf-8
-from __future__ import absolute_import, division, print_function, unicode_literals
+
 
 __license__   = 'GPL v3'
 __copyright__ = '2011, Anthon van der Neut <A.van.der.Neut@ruamel.eu>'
@@ -14,16 +14,14 @@ import sys
 import struct
 
 from calibre.ebooks.djvu.djvubzzdec import BZZDecoder
-from calibre.constants import plugins
 
 
-class DjvuChunk(object):
+class DjvuChunk:
 
     def __init__(self, buf, start, end, align=True, bigendian=True,
             inclheader=False, verbose=0):
-        self.speedup, err = plugins['bzzdec']
-        if self.speedup is None:
-            raise RuntimeError('Failed to load bzzdec plugin: %s' % err)
+        from calibre_extensions import bzzdec
+        self.speedup = bzzdec
         self.subtype = None
         self._subchunks = []
         self.buf = buf
@@ -84,9 +82,9 @@ class DjvuChunk(object):
                 if not res.strip(b'\0'):
                     raise ValueError('TXTz block is completely null')
                 l = 0
-                for x in res[:3]:
+                for x in bytearray(res[:3]):
                     l <<= 8
-                    l += ord(x)
+                    l += x
                 if verbose > 0 and out:
                     print(l, file=out)
                 txtout.write(res[3:3+l])
@@ -94,9 +92,9 @@ class DjvuChunk(object):
         if txtout and self.type == b'TXTa':
             res = self.buf[self.datastart: self.dataend]
             l = 0
-            for x in res[:3]:
+            for x in bytearray(res[:3]):
                 l <<= 8
-                l += ord(x)
+                l += x
             if verbose > 0 and out:
                 print(l, file=out)
             txtout.write(res[3:3+l])
@@ -107,7 +105,7 @@ class DjvuChunk(object):
             schunk.dump(verbose=verbose, indent=indent+1, out=out, txtout=txtout)
 
 
-class DJVUFile(object):
+class DJVUFile:
 
     def __init__(self, instream, verbose=0):
         self.instream = instream
