@@ -1,4 +1,4 @@
-#!/usr/bin/env python2
+#!/usr/bin/env python
 # vim:fileencoding=UTF-8:ts=4:sw=4:sta:et:sts=4:ai
 # License: GPLv3 Copyright: 2011, Kovid Goyal <kovid at kovidgoyal.net>
 from __future__ import absolute_import, division, print_function, unicode_literals
@@ -23,6 +23,14 @@ from calibre.ebooks.metadata.book.base import Metadata
 from calibre.ebooks.metadata.sources.base import Option, Source, fixauthors, fixcase
 from calibre.utils.localization import canonicalize_lang
 from calibre.utils.random_ua import accept_header_for_ua
+from calibre.ebooks.oeb.base import urlquote
+
+
+def iri_quote_plus(url):
+    ans = urlquote(url)
+    if isinstance(ans, bytes):
+        ans = ans.decode('utf-8')
+    return ans.replace('%20', '+')
 
 
 def user_agent_is_ok(ua):
@@ -207,19 +215,33 @@ class Worker(Thread):  # Get details {{{
                 11: ['noviembre'],
                 12: ['diciembre'],
             },
+            'se': {
+                1: ['januari'],
+                2: ['februari'],
+                3: ['mars'],
+                4: ['april'],
+                5: ['maj'],
+                6: ['juni'],
+                7: ['juli'],
+                8: ['augusti'],
+                9: ['september'],
+                10: ['oktober'],
+                11: ['november'],
+                12: ['december'],
+            },
             'jp': {
-                1: [u'1月'],
-                2: [u'2月'],
-                3: [u'3月'],
-                4: [u'4月'],
-                5: [u'5月'],
-                6: [u'6月'],
-                7: [u'7月'],
-                8: [u'8月'],
-                9: [u'9月'],
-                10: [u'10月'],
-                11: [u'11月'],
-                12: [u'12月'],
+                1: ['1月'],
+                2: ['2月'],
+                3: ['3月'],
+                4: ['4月'],
+                5: ['5月'],
+                6: ['6月'],
+                7: ['7月'],
+                8: ['8月'],
+                9: ['9月'],
+                10: ['10月'],
+                11: ['11月'],
+                12: ['12月'],
             },
             'nl': {
                 1: ['januari'], 2: ['februari'], 3: ['maart'], 5: ['mei'], 6: ['juni'], 7: ['juli'], 8: ['augustus'], 10: ['oktober'],
@@ -252,9 +274,14 @@ class Worker(Thread):  # Get details {{{
                     starts-with(text(), "Editor:") or \
                     starts-with(text(), "Editora:") or \
                     starts-with(text(), "Uitgever:") or \
+                    starts-with(text(), "Utgivare:") or \
                     starts-with(text(), "出版社:")]
             '''
-        self.publisher_names = {'Publisher', 'Uitgever', 'Verlag',
+        self.pubdate_xpath = '''
+            descendant::*[starts-with(text(), "Publication Date:") or \
+                    starts-with(text(), "Audible.com Release Date:")]
+        '''
+        self.publisher_names = {'Publisher', 'Uitgever', 'Verlag', 'Utgivare',
                                 'Editore', 'Editeur', 'Editor', 'Editora', '出版社'}
 
         self.language_xpath =    '''
@@ -266,10 +293,11 @@ class Worker(Thread):  # Get details {{{
                 or text() = "Idioma:" \
                 or starts-with(text(), "Langue") \
                 or starts-with(text(), "言語") \
+                or starts-with(text(), "Språk") \
                 or starts-with(text(), "语种")
                 ]
             '''
-        self.language_names = {'Language', 'Sprache',
+        self.language_names = {'Language', 'Sprache', 'Språk',
                                'Lingua', 'Idioma', 'Langue', '言語', 'Taal', '语种'}
 
         self.tags_xpath = '''
@@ -279,25 +307,29 @@ class Worker(Thread):  # Get details {{{
                 text() = "Buscar productos similares por categoría" or
                 text() = "Ricerca articoli simili per categoria" or
                 text() = "Rechercher des articles similaires par rubrique" or
-                text() = "Procure por itens similares por categoria" or
+                text() = "Procure por items similares por categoria" or
                 text() = "関連商品を探す"
             ]/../descendant::ul/li
         '''
 
         self.ratings_pat = re.compile(
-            r'([0-9.]+) ?(out of|von|van|su|étoiles sur|つ星のうち|de un máximo de|de) ([\d\.]+)( (stars|Sternen|stelle|estrellas|estrelas|sterren)){0,1}')
-        self.ratings_pat_cn = re.compile('平均([0-9.]+)')
+            r'([0-9.,]+) ?(out of|von|van|su|étoiles sur|つ星のうち|de un máximo de|de|av) '
+            r'([\d\.]+)( (stars|Sternen|stelle|estrellas|estrelas|sterren|stjärnor)){0,1}'
+        )
+        self.ratings_pat_cn = re.compile('([0-9.]+) 颗星，最多 5 颗星')
+        self.ratings_pat_jp = re.compile(r'\d+つ星のうち([\d\.]+)')
 
         lm = {
-            'eng': ('English', 'Englisch', 'Engels'),
+            'eng': ('English', 'Englisch', 'Engels', 'Engelska'),
             'fra': ('French', 'Français'),
             'ita': ('Italian', 'Italiano'),
             'deu': ('German', 'Deutsch'),
             'spa': ('Spanish', 'Espa\xf1ol', 'Espaniol'),
-            'jpn': ('Japanese', u'日本語'),
+            'jpn': ('Japanese', '日本語'),
             'por': ('Portuguese', 'Português'),
             'nld': ('Dutch', 'Nederlands',),
-            'chs': ('Chinese', u'中文', u'简体中文'),
+            'chs': ('Chinese', '中文', '简体中文'),
+            'swe': ('Swedish', 'Svenska'),
         }
         self.lang_map = {}
         for code, names in lm.items():
@@ -351,7 +383,7 @@ class Worker(Thread):  # Get details {{{
         if self.testing:
             import tempfile
             import uuid
-            with tempfile.NamedTemporaryFile(prefix=(asin or str(uuid.uuid4())) + '_',
+            with tempfile.NamedTemporaryFile(prefix=(asin or type('')(uuid.uuid4())) + '_',
                                              suffix='.html', delete=False) as f:
                 f.write(raw)
             print('Downloaded html for', asin, 'saved in', f.name)
@@ -408,19 +440,23 @@ class Worker(Thread):  # Get details {{{
             self.cover_url = self.parse_cover(root, raw)
         except:
             self.log.exception('Error parsing cover for url: %r' % self.url)
-        if self.cover_url_processor is not None and self.cover_url.startswith('/'):
+        if self.cover_url_processor is not None and self.cover_url and self.cover_url.startswith('/'):
             self.cover_url = self.cover_url_processor(self.cover_url)
         mi.has_cover = bool(self.cover_url)
 
+        detail_bullets = root.xpath('//*[@data-feature-name="detailBullets"]')
         non_hero = tuple(self.selector(
-            'div#bookDetails_container_div div#nonHeroSection'))
-        if non_hero:
-            # New style markup
+            'div#bookDetails_container_div div#nonHeroSection')) or tuple(self.selector(
+                '#productDetails_techSpec_sections'))
+        if detail_bullets:
+            self.parse_detail_bullets(root, mi, detail_bullets[0])
+        elif non_hero:
             try:
                 self.parse_new_details(root, mi, non_hero[0])
             except:
                 self.log.exception(
                     'Failed to parse new-style book details section')
+
         else:
             pd = root.xpath(self.pd_xpath)
             if pd:
@@ -472,8 +508,12 @@ class Worker(Thread):  # Get details {{{
         if self.filter_result(mi, self.log):
             self.result_queue.put(mi)
 
-    def totext(self, elem):
-        return self.tostring(elem, encoding='unicode', method='text').strip()
+    def totext(self, elem, only_printable=False):
+        res = self.tostring(elem, encoding='unicode', method='text')
+        if only_printable:
+            filtered_characters = list(s for s in res if s.isprintable())
+            res = ''.join(filtered_characters).strip()
+        return res
 
     def parse_title(self, root):
 
@@ -511,6 +551,7 @@ class Worker(Thread):  # Get details {{{
                 '#byline .author a.a-link-normal',
                 '#bylineInfo .author .contributorNameID',
                 '#bylineInfo .author a.a-link-normal',
+                '#bylineInfo #bylineContributor',
         ):
             matches = tuple(self.selector(sel))
             if matches:
@@ -536,26 +577,47 @@ class Worker(Thread):  # Get details {{{
             # ratings matches
             x.getparent().remove(x)
 
-        rating_paths = ('//div[@data-feature-name="averageCustomerReviews" or @id="averageCustomerReviews"]',
-                        '//div[@class="jumpBar"]/descendant::span[contains(@class,"asinReviewsSummary")]',
-                        '//div[@class="buying"]/descendant::span[contains(@class,"asinReviewsSummary")]',
-                        '//span[@class="crAvgStars"]/descendant::span[contains(@class,"asinReviewsSummary")]')
+        rating_paths = (
+            '//div[@data-feature-name="averageCustomerReviews" or @id="averageCustomerReviews"]',
+            '//div[@class="jumpBar"]/descendant::span[contains(@class,"asinReviewsSummary")]',
+            '//div[@class="buying"]/descendant::span[contains(@class,"asinReviewsSummary")]',
+            '//span[@class="crAvgStars"]/descendant::span[contains(@class,"asinReviewsSummary")]'
+        )
         ratings = None
         for p in rating_paths:
             ratings = root.xpath(p)
             if ratings:
                 break
+
+        def parse_ratings_text(text):
+            try:
+                m = self.ratings_pat.match(text)
+                return float(m.group(1).replace(',', '.')) / float(m.group(3)) * 5
+            except Exception:
+                pass
+
         if ratings:
-            for elem in ratings[0].xpath('descendant::*[@title]'):
+            ratings = ratings[0]
+            for elem in ratings.xpath('descendant::*[@title]'):
                 t = elem.get('title').strip()
                 if self.domain == 'cn':
                     m = self.ratings_pat_cn.match(t)
                     if m is not None:
                         return float(m.group(1))
-                else:
-                    m = self.ratings_pat.match(t)
+                elif self.domain == 'jp':
+                    m = self.ratings_pat_jp.match(t)
                     if m is not None:
-                        return float(m.group(1)) / float(m.group(3)) * 5
+                        return float(m.group(1))
+                else:
+                    ans = parse_ratings_text(t)
+                    if ans is not None:
+                        return ans
+            for elem in ratings.xpath('descendant::span[@class="a-icon-alt"]'):
+                t = self.tostring(
+                    elem, encoding='unicode', method='text', with_tail=False).strip()
+                ans = parse_ratings_text(t)
+                if ans is not None:
+                    return ans
 
     def _render_comments(self, desc):
         from calibre.library.comments import sanitize_comments_html
@@ -634,6 +696,10 @@ class Worker(Thread):  # Get details {{{
                 except Exception as e:
                     self.log.warn(
                         'Parsing of obfuscated product description failed with error: %s' % as_unicode(e))
+            else:
+                desc = root.xpath('//div[@id="productDescription_fullView"]')
+                if desc:
+                    ans += self._render_comments(desc[0])
 
         return ans
 
@@ -657,6 +723,23 @@ class Worker(Thread):  # Get details {{{
                             s[0], encoding='unicode', method='text', with_tail=False).strip()
                         if series:
                             ans = (series, series_index)
+        else:
+            series = root.xpath('//div[@id="seriesBullet_feature_div"]')
+            if series:
+                series = series[0]
+                spans = series.xpath('descendant::span')
+                if spans:
+                    span = spans[0]
+                    b = span.xpath('./b')
+                    a = span.xpath('./a')
+                    if a and b:
+                        series = self.tostring(a[0], encoding='unicode', method='text', with_tail=False).strip()
+                        if series:
+                            raw = self.tostring(b[0], encoding='unicode', method='text', with_tail=False).strip()
+                            m = re.search(r'[0-9.]+', raw)
+                            if m is not None:
+                                ans = (series, float(m.group()))
+
         # This is found on Kindle edition pages on amazon.com
         if ans == (None, None):
             for span in root.xpath('//div[@id="aboutEbooksSection"]//li/span'):
@@ -789,36 +872,50 @@ class Worker(Thread):  # Get details {{{
                 if url:
                     return url
 
+    def parse_detail_bullets(self, root, mi, container):
+        ul = next(self.selector('.detail-bullet-list', root=container))
+        for span in self.selector('.a-list-item', root=ul):
+            cells = span.xpath('./span')
+            if len(cells) >= 2:
+                self.parse_detail_cells(mi, cells[0], cells[1])
+
     def parse_new_details(self, root, mi, non_hero):
         table = non_hero.xpath('descendant::table')[0]
         for tr in table.xpath('descendant::tr'):
-            cells = tr.xpath('descendant::td')
+            cells = tr.xpath('descendant::*[local-name()="td" or local-name()="th"]')
             if len(cells) == 2:
-                name = self.totext(cells[0])
-                val = self.totext(cells[1])
-                if not val:
-                    continue
-                if name in self.language_names:
-                    ans = self.lang_map.get(val, None)
-                    if not ans:
-                        ans = canonicalize_lang(val)
-                    if ans:
-                        mi.language = ans
-                elif name in self.publisher_names:
-                    pub = val.partition(';')[0].partition('(')[0].strip()
-                    if pub:
-                        mi.publisher = pub
-                    date = val.rpartition('(')[-1].replace(')', '').strip()
-                    try:
-                        from calibre.utils.date import parse_only_date
-                        date = self.delocalize_datestr(date)
-                        mi.pubdate = parse_only_date(date, assume_utc=True)
-                    except:
-                        self.log.exception('Failed to parse pubdate: %s' % val)
-                elif name in {'ISBN', 'ISBN-10', 'ISBN-13'}:
-                    ans = check_isbn(val)
-                    if ans:
-                        self.isbn = mi.isbn = ans
+                self.parse_detail_cells(mi, cells[0], cells[1])
+
+    def parse_detail_cells(self, mi, c1, c2):
+        name = self.totext(c1, only_printable=True).strip().strip(':').strip()
+        val = self.totext(c2)
+        if not val:
+            return
+        if name in self.language_names:
+            ans = self.lang_map.get(val, None)
+            if not ans:
+                ans = canonicalize_lang(val)
+            if ans:
+                mi.language = ans
+        elif name in self.publisher_names:
+            pub = val.partition(';')[0].partition('(')[0].strip()
+            if pub:
+                mi.publisher = pub
+            date = val.rpartition('(')[-1].replace(')', '').strip()
+            try:
+                from calibre.utils.date import parse_only_date
+                date = self.delocalize_datestr(date)
+                mi.pubdate = parse_only_date(date, assume_utc=True)
+            except:
+                self.log.exception('Failed to parse pubdate: %s' % val)
+        elif name in {'ISBN', 'ISBN-10', 'ISBN-13'}:
+            ans = check_isbn(val)
+            if ans:
+                self.isbn = mi.isbn = ans
+        elif name in {'Publication date'}:
+            from calibre.utils.date import parse_only_date
+            date = self.delocalize_datestr(val)
+            mi.pubdate = parse_only_date(date, assume_utc=True)
 
     def parse_isbn(self, pd):
         items = pd.xpath(
@@ -839,13 +936,24 @@ class Worker(Thread):  # Get details {{{
                 return ans.partition('(')[0].strip()
 
     def parse_pubdate(self, pd):
+        from calibre.utils.date import parse_only_date
+        for x in reversed(pd.xpath(self.pubdate_xpath)):
+            if x.tail:
+                date = x.tail.strip()
+                date = self.delocalize_datestr(date)
+                try:
+                    return parse_only_date(date, assume_utc=True)
+                except Exception:
+                    pass
         for x in reversed(pd.xpath(self.publisher_xpath)):
             if x.tail:
-                from calibre.utils.date import parse_only_date
                 ans = x.tail
                 date = ans.rpartition('(')[-1].replace(')', '').strip()
                 date = self.delocalize_datestr(date)
-                return parse_only_date(date, assume_utc=True)
+                try:
+                    return parse_only_date(date, assume_utc=True)
+                except Exception:
+                    pass
 
     def parse_language(self, pd):
         for x in reversed(pd.xpath(self.language_xpath)):
@@ -863,7 +971,7 @@ class Worker(Thread):  # Get details {{{
 class Amazon(Source):
 
     name = 'Amazon.com'
-    version = (1, 2, 10)
+    version = (1, 2, 19)
     minimum_calibre_version = (2, 82, 0)
     description = _('Downloads metadata and covers from Amazon')
 
@@ -888,6 +996,7 @@ class Amazon(Source):
         'nl': _('Netherlands'),
         'cn': _('China'),
         'ca': _('Canada'),
+        'se': _('Sweden'),
     }
 
     SERVERS = {
@@ -995,6 +1104,7 @@ class Amazon(Source):
             'uk':  'https://www.amazon.co.uk/',
             'au':  'https://www.amazon.com.au/',
             'br':  'https://www.amazon.com.br/',
+            'jp':  'https://www.amazon.co.jp/',
         }.get(domain, 'https://www.amazon.%s/' % domain)
 
     def _get_book_url(self, identifiers):  # {{{
@@ -1076,9 +1186,9 @@ class Amazon(Source):
     def create_query(self, log, title=None, authors=None, identifiers={},  # {{{
                      domain=None, for_amazon=True):
         try:
-            from urllib.parse import urlencode
+            from urllib.parse import urlencode, unquote_plus
         except ImportError:
-            from urllib import urlencode
+            from urllib import urlencode, unquote_plus
         if domain is None:
             domain = self.domain
 
@@ -1132,28 +1242,25 @@ class Amazon(Source):
         if not for_amazon:
             return terms, domain
 
-        # magic parameter to enable Japanese Shift_JIS encoding.
-        if domain == 'jp':
-            q['__mk_ja_JP'] = u'カタカナ'
         if domain == 'nl':
-            q['__mk_nl_NL'] = u'ÅMÅŽÕÑ'
+            q['__mk_nl_NL'] = 'ÅMÅŽÕÑ'
             if 'field-keywords' not in q:
                 q['field-keywords'] = ''
             for f in 'field-isbn field-title field-author'.split():
                 q['field-keywords'] += ' ' + q.pop(f, '')
             q['field-keywords'] = q['field-keywords'].strip()
 
-        if domain == 'jp':
-            encode_to = 'Shift_JIS'
-        elif domain == 'nl' or domain == 'cn':
-            encode_to = 'utf-8'
-        else:
-            encode_to = 'latin1'
-        encoded_q = dict([(x.encode(encode_to, 'ignore'), y.encode(encode_to,
-                                                                   'ignore')) for x, y in
-                          q.items()])
+        encoded_q = dict([(x.encode('utf-8', 'ignore'), y.encode(
+            'utf-8', 'ignore')) for x, y in q.items()])
+        url_query = urlencode(encoded_q)
+        # amazon's servers want IRIs with unicode characters not percent esaped
+        parts = []
+        for x in url_query.split(b'&' if isinstance(url_query, bytes) else '&'):
+            k, v = x.split(b'=' if isinstance(x, bytes) else '=', 1)
+            parts.append('{}={}'.format(iri_quote_plus(unquote_plus(k)), iri_quote_plus(unquote_plus(v))))
+        url_query = '&'.join(parts)
         url = 'https://www.amazon.%s/s/?' % self.get_website_domain(
-            domain) + urlencode(encoded_q)
+            domain) + url_query
         return url, domain
 
     # }}}
@@ -1430,9 +1537,13 @@ class Amazon(Source):
         if not self.use_search_engine:
             return True
         if title is not None:
-            tokens = {icu_lower(x).rstrip(':') for x in title.split() if len(x) > 3}
+
+            def tokenize_title(x):
+                return icu_lower(x).replace("'", '').replace('"', '').rstrip(':')
+
+            tokens = {tokenize_title(x) for x in title.split() if len(x) > 3}
             if tokens:
-                result_tokens = {icu_lower(x).rstrip(':') for x in mi.title.split()}
+                result_tokens = {tokenize_title(x) for x in mi.title.split()}
                 if not tokens.intersection(result_tokens):
                     log('Ignoring result:', mi.title, 'as its title does not match')
                     return False
@@ -1504,7 +1615,7 @@ def manual_tests(domain, **kw):  # {{{
         (   # Paperback with series
             {'identifiers': {'amazon': '1423146786'}},
             [title_test('The Heroes of Olympus, Book Five The Blood of Olympus',
-                        exact=True), series_test('Heroes of Olympus', 5)]
+                        exact=True), series_test('The Heroes of Olympus', 5)]
         ),
 
         (   # Kindle edition with series
@@ -1540,7 +1651,7 @@ def manual_tests(domain, **kw):  # {{{
         (  # No specific problems
             {'identifiers': {'isbn': '0743273567'}},
             [title_test('The great gatsby', exact=True),
-             authors_test(['F. Scott Fitzgerald'])]
+             authors_test(['Francis Scott Fitzgerald'])]
         ),
 
     ]
@@ -1548,6 +1659,15 @@ def manual_tests(domain, **kw):  # {{{
     # }}}
 
     all_tests['de'] = [  # {{{
+        (  # umlaut in title/authors
+            {'title': 'Flüsternde Wälder',
+             'authors': ['Nicola Förg']},
+            [title_test('Flüsternde Wälder'),
+             authors_test(['Nicola Förg'])
+             ]
+        ),
+
+
         (
             {'identifiers': {'isbn': '9783453314979'}},
             [title_test('Die letzten Wächter: Roman',
@@ -1601,21 +1721,31 @@ def manual_tests(domain, **kw):  # {{{
         ),
     ]  # }}}
 
+    all_tests['se'] = [  # {{{
+        (
+            {'identifiers': {'isbn': '9780552140287'}},
+            [title_test('Men At Arms: A Discworld Novel: 14',
+                        exact=False), authors_test(['Terry Pratchett'])
+             ]
+
+        ),
+    ]  # }}}
+
     all_tests['jp'] = [  # {{{
         (  # Adult filtering test
             {'identifiers': {'isbn': '4799500066'}},
-            [title_test(u'Ｂｉｔｃｈ Ｔｒａｐ'), ]
+            [title_test('Ｂｉｔｃｈ Ｔｒａｐ'), ]
         ),
 
         (  # isbn -> title, authors
             {'identifiers': {'isbn': '9784101302720'}},
-            [title_test(u'精霊の守り人',
-                        exact=True), authors_test([u'上橋 菜穂子'])
+            [title_test('精霊の守り人',
+                        exact=True), authors_test(['上橋 菜穂子'])
              ]
         ),
         (  # title, authors -> isbn (will use Shift_JIS encoding in query.)
-            {'title': u'考えない練習',
-             'authors': [u'小池 龍之介']},
+            {'title': '考えない練習',
+             'authors': ['小池 龍之介']},
             [isbn_test('9784093881067'), ]
         ),
     ]  # }}}
