@@ -1,8 +1,6 @@
 '''
 Transform XHTML/OPS-ish content into Mobipocket HTML 3.2.
 '''
-from __future__ import with_statement
-from __future__ import print_function
 
 __license__   = 'GPL v3'
 __copyright__ = '2008, Marshall T. Vandegrift <llasram@gmail.cam>'
@@ -17,7 +15,7 @@ from calibre.ebooks.oeb.stylizer import Stylizer
 from calibre.ebooks.oeb.transforms.flatcss import KeyMapper
 from calibre.ebooks.mobi.utils import convert_color_for_font_tag
 from calibre.utils.imghdr import identify
-from polyglot.builtins import unicode_type, string_or_bytes
+from polyglot.builtins import string_or_bytes
 
 MBP_NS = 'http://mobipocket.com/ns/mbp'
 
@@ -51,15 +49,25 @@ def asfloat(value):
     return float(value)
 
 
+def convert_margin(style, which):
+    # percentage values come out too large when the user uses a non kindle
+    # output profile like the tablet profile
+    ans = asfloat(style[which])
+    raw = style._get(which)
+    if isinstance(raw, str) and '%' in raw:
+        ans = min(style._unit_convert(raw, base=600), ans)
+    return ans
+
+
 def isspace(text):
     if not text:
         return True
-    if u'\xa0' in text:
+    if '\xa0' in text:
         return False
     return text.isspace()
 
 
-class BlockState(object):
+class BlockState:
 
     def __init__(self, body):
         self.body = body
@@ -74,7 +82,7 @@ class BlockState(object):
         self.content = False
 
 
-class FormatState(object):
+class FormatState:
 
     def __init__(self):
         self.rendered = False
@@ -113,7 +121,7 @@ class FormatState(object):
         return not self.__eq__(other)
 
 
-class MobiMLizer(object):
+class MobiMLizer:
 
     def __init__(self, ignore_tables=False):
         self.ignore_tables = ignore_tables
@@ -124,7 +132,7 @@ class MobiMLizer(object):
         self.log = self.oeb.logger
         self.opts = context
         self.profile = profile = context.dest
-        self.fnums = fnums = dict((v, k) for k, v in profile.fnums.items())
+        self.fnums = fnums = {v: k for k, v in profile.fnums.items()}
         self.fmap = KeyMapper(profile.fbase, profile.fbase, fnums.keys())
         self.mobimlize_spine()
 
@@ -139,7 +147,7 @@ class MobiMLizer(object):
             self.mobimlize_elem(body, stylizer, BlockState(nbody),
                                 [FormatState()])
             item.data = nroot
-            # print etree.tostring(nroot)
+            # print(etree.tostring(nroot))
 
     def mobimlize_font(self, ptsize):
         return self.fnums[self.fmap[ptsize]]
@@ -153,12 +161,12 @@ class MobiMLizer(object):
         return "%dem" % int(round(ptsize / embase))
 
     def preize_text(self, text, pre_wrap=False):
-        text = unicode_type(text)
+        text = str(text)
         if pre_wrap:
             # Replace n consecutive spaces with n-1 NBSP + space
-            text = re.sub(r' {2,}', lambda m:(u'\xa0'*(len(m.group())-1) + u' '), text)
+            text = re.sub(r' {2,}', lambda m:('\xa0'*(len(m.group())-1) + ' '), text)
         else:
-            text = text.replace(u' ', u'\xa0')
+            text = text.replace(' ', '\xa0')
 
         text = text.replace('\r\n', '\n')
         text = text.replace('\r', '\n')
@@ -210,7 +218,7 @@ class MobiMLizer(object):
                 para = wrapper
                 emleft = int(round(left / self.profile.fbase)) - ems
                 emleft = min((emleft, 10))
-                while emleft > ems/2.0:
+                while emleft > ems / 2:
                     para = etree.SubElement(para, XHTML('blockquote'))
                     emleft -= ems
             else:
@@ -230,7 +238,7 @@ class MobiMLizer(object):
                 while vspace > 0:
                     wrapper.addprevious(etree.Element(XHTML('br')))
                     vspace -= 1
-            if istate.halign != 'auto' and isinstance(istate.halign, (bytes, unicode_type)):
+            if istate.halign != 'auto' and isinstance(istate.halign, (bytes, str)):
                 if isinstance(istate.halign, bytes):
                     istate.halign = istate.halign.decode('utf-8')
                 para.attrib['align'] = istate.halign
@@ -363,7 +371,10 @@ class MobiMLizer(object):
             bstate.para = None
             istate.halign = style['text-align']
             rawti = style._get('text-indent')
-            istate.indent = style['text-indent']
+            try:
+                istate.indent = style['text-indent']
+            except Exception:
+                istate.indent = 0
             if hasattr(rawti, 'strip') and '%' in rawti:
                 # We have a percentage text indent, these can come out looking
                 # too large if the user chooses a wide output profile like
@@ -372,12 +383,12 @@ class MobiMLizer(object):
             if style['margin-left'] == 'auto' \
                and style['margin-right'] == 'auto':
                 istate.halign = 'center'
-            margin = asfloat(style['margin-left'])
+            margin = convert_margin(style, 'margin-left')
             padding = asfloat(style['padding-left'])
             if tag != 'body':
                 left = margin + padding
             istate.left += left
-            vmargin = asfloat(style['margin-top'])
+            vmargin = convert_margin(style, 'margin-top')
             bstate.vmargin = max((bstate.vmargin, vmargin))
             vpadding = asfloat(style['padding-top'])
             if vpadding > 0:
@@ -385,22 +396,22 @@ class MobiMLizer(object):
                 bstate.vmargin = 0
                 bstate.vpadding += vpadding
         elif not istate.href:
-            margin = asfloat(style['margin-left'])
+            margin = convert_margin(style, 'margin-left')
             padding = asfloat(style['padding-left'])
             lspace = margin + padding
             if lspace > 0:
                 spaces = int(round((lspace * 3) / style['font-size']))
-                elem.text = (u'\xa0' * spaces) + (elem.text or '')
-            margin = asfloat(style['margin-right'])
+                elem.text = ('\xa0' * spaces) + (elem.text or '')
+            margin = convert_margin(style, 'margin-right')
             padding = asfloat(style['padding-right'])
             rspace = margin + padding
             if rspace > 0:
                 spaces = int(round((rspace * 3) / style['font-size']))
                 if len(elem) == 0:
-                    elem.text = (elem.text or '') + (u'\xa0' * spaces)
+                    elem.text = (elem.text or '') + ('\xa0' * spaces)
                 else:
                     last = elem[-1]
-                    last.text = (last.text or '') + (u'\xa0' * spaces)
+                    last.text = (last.text or '') + ('\xa0' * spaces)
         if bstate.content and style['page-break-before'] in PAGE_BREAKS:
             bstate.pbreak = True
         istate.fsize = self.mobimlize_font(style['font-size'])
@@ -446,7 +457,7 @@ class MobiMLizer(object):
                         # See #7520 for test case
                         try:
                             pixs = int(round(float(value) /
-                                (72./self.profile.dpi)))
+                                (72/self.profile.dpi)))
                         except:
                             continue
                         result = str(pixs)
@@ -469,7 +480,7 @@ class MobiMLizer(object):
                             istate.attrib['width'] = str(width)
                             istate.attrib['height'] = str(height)
                         else:
-                            ar = float(width)/float(height)
+                            ar = width / height
                             if 'width' not in istate.attrib:
                                 try:
                                     width = int(istate.attrib['height'])*ar
@@ -515,11 +526,11 @@ class MobiMLizer(object):
             t = elem.text
             if not t:
                 t = ''
-            elem.text = u'\u201c' + t
+            elem.text = '\u201c' + t
             t = elem.tail
             if not t:
                 t = ''
-            elem.tail = u'\u201d' + t
+            elem.tail = '\u201d' + t
         text = None
         if elem.text:
             if istate.preserve or istate.pre_wrap:
@@ -602,7 +613,7 @@ class MobiMLizer(object):
             bstate.pbreak = True
         if isblock:
             para = bstate.para
-            if para is not None and para.text == u'\xa0' and len(para) < 1:
+            if para is not None and para.text == '\xa0' and len(para) < 1:
                 if style.height > 2:
                     para.getparent().replace(para, etree.Element(XHTML('br')))
                 else:
@@ -610,7 +621,7 @@ class MobiMLizer(object):
                     para.getparent().remove(para)
             bstate.para = None
             bstate.istate = None
-            vmargin = asfloat(style['margin-bottom'])
+            vmargin = convert_margin(style, 'margin-bottom')
             bstate.vmargin = max((bstate.vmargin, vmargin))
             vpadding = asfloat(style['padding-bottom'])
             if vpadding > 0:
