@@ -1,5 +1,5 @@
 # vim:fileencoding=UTF-8:ts=4:sw=4:sta:et:sts=4:fdm=marker:ai
-from __future__ import absolute_import, division, print_function, unicode_literals
+
 
 __license__   = 'GPL v3'
 __copyright__ = '2013, Kovid Goyal <kovid at kovidgoyal.net>'
@@ -10,7 +10,7 @@ import shutil, re, os
 from calibre.ebooks.oeb.base import OPF, OEB_DOCS, XPath, XLINK, xml2text
 from calibre.ebooks.oeb.polish.replace import replace_links, get_recommended_folders
 from calibre.utils.imghdr import identify
-from polyglot.builtins import iteritems, unicode_type
+from polyglot.builtins import iteritems
 
 
 def set_azw3_cover(container, cover_path, report, options=None):
@@ -384,8 +384,8 @@ def create_epub_cover(container, cover_path, existing_image, options=None):
             ar = 'xMidYMid meet' if keep_aspect else 'none'
             templ = CoverManager.SVG_TEMPLATE.replace('__ar__', ar)
             templ = templ.replace('__viewbox__', '0 0 %d %d'%(width, height))
-            templ = templ.replace('__width__', unicode_type(width))
-            templ = templ.replace('__height__', unicode_type(height))
+            templ = templ.replace('__width__', str(width))
+            templ = templ.replace('__height__', str(height))
     folder = recommended_folders[tname]
     if folder:
         tname = folder + '/' + tname
@@ -440,7 +440,20 @@ def remove_cover_image_in_page(container, page, cover_images):
         break
 
 
-def set_epub_cover(container, cover_path, report, options=None):
+def has_epub_cover(container):
+    if find_cover_image(container):
+        return True
+    if find_cover_page(container):
+        return True
+    spine_items = tuple(container.spine_items)
+    if spine_items:
+        candidate = container.abspath_to_name(spine_items[0])
+        if find_cover_image_in_page(container, candidate) is not None:
+            return True
+    return False
+
+
+def set_epub_cover(container, cover_path, report, options=None, image_callback=None):
     existing_image = options is not None and options.get('existing_image', False)
     if existing_image:
         existing_image = cover_path
@@ -455,6 +468,7 @@ def set_epub_cover(container, cover_path, report, options=None):
     # TODO: Handle possible_removals and also iterate over links in the removed
     # pages and handle possibly removing stylesheets referred to by them.
 
+    image_callback_called = False
     spine_items = tuple(container.spine_items)
     if cover_page is None and spine_items:
         # Check if the first item in the spine is a simple cover wrapper
@@ -487,12 +501,18 @@ def set_epub_cover(container, cover_path, report, options=None):
         if wrapped_image is not None:
             # The cover page is a simple wrapper around a single cover image,
             # we can remove it safely.
-            log('Existing cover page is a simple wrapper, removing it')
+            log('Existing cover page {} is a simple wrapper, removing it'.format(cover_page))
             container.remove_item(cover_page)
             if wrapped_image != existing_image:
+                if image_callback is not None and not image_callback_called:
+                    image_callback(cover_image, wrapped_image)
+                    image_callback_called = True
                 container.remove_item(wrapped_image)
             updated = True
 
+    if image_callback is not None and not image_callback_called:
+        image_callback_called = True
+        image_callback(cover_image, wrapped_image)
     if cover_image and cover_image != wrapped_image:
         # Remove the old cover image
         if cover_image != existing_image:
