@@ -17,7 +17,7 @@ htmlstrip = re.compile("<"  # open
 
 def clean_attributes(html):
     while htmlstrip.search(html):
-        html = htmlstrip.sub('<\\1\\2>', html)
+        html = htmlstrip.sub(r'<\1\2>', html)
     return html
 
 
@@ -28,6 +28,7 @@ def normalize_spaces(s):
     characters with a single space"""
     return ' '.join(s.split())
 
+
 html_cleaner = Cleaner(scripts=True, javascript=True, comments=True,
                   style=True, links=True, meta=False, add_nofollow=False,
                   page_structure=False, processing_instructions=True, embedded=False,
