@@ -6,28 +6,31 @@ __license__   = 'GPL v3'
 __copyright__ = '2010, Kovid Goyal <kovid@kovidgoyal.net>'
 __docformat__ = 'restructuredtext en'
 
-import os, shutil, copy
+import copy
+import os
+import shutil
 from functools import partial
 from io import BytesIO
+from qt.core import (
+    QAction, QApplication, QDialog, QIcon, QMenu, QMimeData, QModelIndex, QTimer, QUrl
+)
 
-from PyQt5.Qt import QMenu, QModelIndex, QTimer, QIcon, QApplication, QMimeData, QDialog
-
-from calibre.gui2 import error_dialog, Dispatcher, question_dialog, gprefs
-from calibre.gui2.dialogs.metadata_bulk import MetadataBulkDialog
-from calibre.gui2.dialogs.confirm_delete import confirm
-from calibre.gui2.dialogs.device_category_editor import DeviceCategoryEditor
-from calibre.gui2.actions import InterfaceAction
+from calibre.db.errors import NoSuchFormat
 from calibre.ebooks.metadata import authors_to_string
 from calibre.ebooks.metadata.book.base import Metadata
 from calibre.ebooks.metadata.opf2 import OPF, metadata_to_opf
+from calibre.ebooks.metadata.sources.prefs import msprefs
+from calibre.gui2 import Dispatcher, error_dialog, gprefs, question_dialog
+from calibre.gui2.actions import InterfaceAction
+from calibre.gui2.actions.show_quickview import get_quickview_action_plugin
+from calibre.gui2.dialogs.confirm_delete import confirm
+from calibre.gui2.dialogs.device_category_editor import DeviceCategoryEditor
+from calibre.gui2.dialogs.metadata_bulk import MetadataBulkDialog
+from calibre.library.comments import merge_comments
 from calibre.utils.config import tweaks
 from calibre.utils.date import is_date_undefined
 from calibre.utils.icu import sort_key
-from calibre.db.errors import NoSuchFormat
-from calibre.library.comments import merge_comments
-from calibre.ebooks.metadata.sources.prefs import msprefs
-from calibre.gui2.actions.show_quickview import get_quickview_action_plugin
-from polyglot.builtins import iteritems, unicode_type, map
+from polyglot.builtins import iteritems
 
 
 class EditMetadataAction(InterfaceAction):
@@ -99,6 +102,57 @@ class EditMetadataAction(InterfaceAction):
         self.action_merge.setMenu(mb)
 
         self.qaction.triggered.connect(self.edit_metadata)
+        ac = QAction(_('Copy URL to show book in calibre'), self.gui)
+        ac.setToolTip(_('Copy URLs to show the currently selected books in calibre, to the system clipboard'))
+        ac.triggered.connect(self.copy_show_link)
+        self.gui.addAction(ac)
+        self.gui.keyboard.register_shortcut(
+            self.unique_name + ' - ' + 'copy_show_book',
+            ac.text(), description=ac.toolTip(),
+            action=ac, group=self.action_spec[0])
+        ac = QAction(_('Copy URL to open book in calibre'), self.gui)
+        ac.triggered.connect(self.copy_view_link)
+        ac.setToolTip(_('Copy URLs to open the currently selected books in calibre, to the system clipboard'))
+        self.gui.addAction(ac)
+        self.gui.keyboard.register_shortcut(
+            self.unique_name + ' - ' + 'copy_view_book',
+            ac.text(), description=ac.toolTip(),
+            action=ac, group=self.action_spec[0])
+
+    def _copy_links(self, lines):
+        urls = QUrl.fromStringList(lines)
+        cb = QApplication.instance().clipboard()
+        md = QMimeData()
+        md.setText('\n'.join(lines))
+        md.setUrls(urls)
+        cb.setMimeData(md)
+
+    def copy_show_link(self):
+        db = self.gui.current_db
+        ids = [db.id(row.row()) for row in self.gui.library_view.selectionModel().selectedRows()]
+        db = db.new_api
+        library_id = getattr(db, 'server_library_id', None)
+        if not library_id or not ids:
+            return
+        lines = [f'calibre://show-book/{library_id}/{book_id}' for book_id in ids]
+        self._copy_links(lines)
+
+    def copy_view_link(self):
+        from calibre.gui2.actions.view import preferred_format
+        db = self.gui.current_db
+        ids = [db.id(row.row()) for row in self.gui.library_view.selectionModel().selectedRows()]
+        db = db.new_api
+        library_id = getattr(db, 'server_library_id', None)
+        if not library_id or not ids:
+            return
+        lines = []
+        for book_id in ids:
+            formats = db.new_api.formats(book_id, verify_formats=True)
+            if formats:
+                fmt = preferred_format(formats)
+                lines.append(f'calibre://view-book/{library_id}/{book_id}/{fmt}')
+        if lines:
+            self._copy_links(lines)
 
     def location_selected(self, loc):
         enabled = loc == 'library'
@@ -119,7 +173,7 @@ class EditMetadataAction(InterfaceAction):
         book_id = db.id(rows[0].row())
         mi = db.new_api.get_metadata(book_id)
         md = QMimeData()
-        md.setText(unicode_type(mi))
+        md.setText(str(mi))
         md.setData('application/calibre-book-metadata', bytearray(metadata_to_opf(mi, default_lang='und')))
         img = db.new_api.cover(book_id, as_image=True)
         if img:
@@ -174,8 +228,8 @@ class EditMetadataAction(InterfaceAction):
                             _('No books selected'), show=True)
             db = self.gui.library_view.model().db
             ids = [db.id(row.row()) for row in rows]
-        from calibre.gui2.metadata.bulk_download import start_download
         from calibre.ebooks.metadata.sources.update import update_sources
+        from calibre.gui2.metadata.bulk_download import start_download
         update_sources()
         start_download(self.gui, ids,
                 Dispatcher(self.metadata_downloaded),
@@ -648,7 +702,7 @@ class EditMetadataAction(InterfaceAction):
                 if not dest_mi.comments:
                     dest_mi.comments = src_mi.comments
                 else:
-                    dest_mi.comments = unicode_type(dest_mi.comments) + '\n\n' + unicode_type(src_mi.comments)
+                    dest_mi.comments = str(dest_mi.comments) + '\n\n' + str(src_mi.comments)
             if src_mi.title and (not dest_mi.title or dest_mi.title == _('Unknown')):
                 dest_mi.title = src_mi.title
             if (src_mi.authors and src_mi.authors[0] != _('Unknown')) and (not dest_mi.authors or dest_mi.authors[0] == _('Unknown')):
@@ -701,7 +755,7 @@ class EditMetadataAction(InterfaceAction):
                     if not dest_value:
                         db.set_custom(dest_id, src_value, num=colnum)
                     else:
-                        dest_value = unicode_type(dest_value) + '\n\n' + unicode_type(src_value)
+                        dest_value = str(dest_value) + '\n\n' + str(src_value)
                         db.set_custom(dest_id, dest_value, num=colnum)
                 if (dt in {'bool', 'int', 'float', 'rating', 'datetime'} and dest_value is None):
                     db.set_custom(dest_id, src_value, num=colnum)
@@ -727,7 +781,7 @@ class EditMetadataAction(InterfaceAction):
             to_rename = d.to_rename  # dict of new text to old ids
             to_delete = d.to_delete  # list of ids
             for old_id, new_name in iteritems(to_rename):
-                model.rename_collection(old_id, new_name=unicode_type(new_name))
+                model.rename_collection(old_id, new_name=str(new_name))
             for item in to_delete:
                 model.delete_collection_using_id(item)
             self.gui.upload_collections(model.db, view=view, oncard=oncard)
@@ -849,7 +903,7 @@ class EditMetadataAction(InterfaceAction):
 
             error_dialog(self.gui, _('Some failures'),
                 _('Failed to apply updated metadata for some books'
-                    ' in your library. Click "Show Details" to see '
+                    ' in your library. Click "Show details" to see '
                     'details.'), det_msg='\n\n'.join(msg), show=True)
         changed_books = len(self.applied_ids or ())
         self.refresh_gui(self.applied_ids)
@@ -899,8 +953,8 @@ class EditMetadataAction(InterfaceAction):
             self.refresh_books_after_metadata_edit(affected_books)
 
     def set_cover_from_format(self, book_id, fmt):
-        from calibre.utils.config import prefs
         from calibre.ebooks.metadata.meta import get_metadata
+        from calibre.utils.config import prefs
         fmt = fmt.lower()
         cdata = None
         db = self.gui.current_db.new_api
