@@ -3,7 +3,7 @@
 # License: GPLv3 Copyright: 2016, Kovid Goyal <kovid at kovidgoyal.net>
 
 
-from PyQt5.Qt import (
+from qt.core import (
     QWidget, QVBoxLayout, QHBoxLayout, QLabel, QComboBox, QLineEdit,
     QPushButton, QSize, pyqtSignal, QMenu, QDialogButtonBox, QTextCursor
 )
@@ -17,7 +17,7 @@ from calibre.gui2.tag_mapper import (
 from calibre.gui2.widgets2 import Dialog
 from calibre.utils.config import JSONConfig
 from calibre.utils.localization import localize_user_manual_link
-from polyglot.builtins import iteritems, unicode_type
+from polyglot.builtins import iteritems
 
 
 class RuleEdit(QWidget):  # {{{
@@ -132,14 +132,14 @@ class RuleEdit(QWidget):  # {{{
     def rule(self, rule):
         def sc(name):
             c = getattr(self, name)
-            idx = c.findData(unicode_type(rule.get(name, '')))
+            idx = c.findData(str(rule.get(name, '')))
             if idx < 0:
                 idx = 0
             c.setCurrentIndex(idx)
         sc('action'), sc('match_type')
-        self.property.setText(unicode_type(rule.get('property', '')).strip())
-        self.query.setText(unicode_type(rule.get('query', '')).strip())
-        self.action_data.setText(unicode_type(rule.get('action_data', '')).strip())
+        self.property.setText(str(rule.get('property', '')).strip())
+        self.query.setText(str(rule.get('query', '')).strip())
+        self.action_data.setText(str(rule.get('action_data', '')).strip())
         self.update_state()
 
     def validate(self):
