@@ -1,15 +1,14 @@
-from __future__ import with_statement
 __license__   = 'GPL v3'
 __copyright__ = '2008, Kovid Goyal <kovid at kovidgoyal.net>'
 
 # Imports {{{
-import os, traceback, Queue, time, cStringIO, re, sys, weakref
+import os, traceback, time, io, re, sys, weakref
 from threading import Thread, Event
 
-from PyQt5.Qt import (
+from qt.core import (
     QMenu, QAction, QActionGroup, QIcon, Qt, pyqtSignal, QDialog,
     QObject, QVBoxLayout, QDialogButtonBox, QCursor, QCoreApplication,
-    QApplication, QEventLoop)
+    QApplication, QEventLoop, QTimer)
 
 from calibre.customize.ui import (available_input_formats, available_output_formats,
     device_plugins, disabled_device_plugins)
@@ -24,7 +23,7 @@ from calibre.gui2 import (config, error_dialog, Dispatcher, dynamic,
         warning_dialog, info_dialog, choose_dir, FunctionDispatcher,
         show_restart_warning, gprefs, question_dialog)
 from calibre.ebooks.metadata import authors_to_string
-from calibre import preferred_encoding, prints, force_unicode, as_unicode, sanitize_file_name2
+from calibre import preferred_encoding, prints, force_unicode, as_unicode, sanitize_file_name
 from calibre.utils.filenames import ascii_filename
 from calibre.devices.errors import (FreeSpaceError, WrongDestinationError,
         BlacklistedDevice)
@@ -34,6 +33,8 @@ from calibre.utils.config import tweaks, device_prefs
 from calibre.utils.img import scale_image
 from calibre.library.save_to_disk import find_plugboard
 from calibre.ptempfile import PersistentTemporaryFile, force_unicode as filename_to_unicode
+from polyglot.builtins import string_or_unicode
+from polyglot import queue
 # }}}
 
 
@@ -56,8 +57,7 @@ class DeviceJob(BaseJob):  # {{{
 
     def start_work(self):
         if DEBUG:
-            prints('Job:', self.id, self.description, 'started',
-                safe_encode=True)
+            prints('Job:', self.id, self.description, 'started')
         self.start_time = time.time()
         self.job_manager.changed_queue.put(self)
 
@@ -66,7 +66,7 @@ class DeviceJob(BaseJob):  # {{{
         self.percent = 1
         if DEBUG:
             prints('DeviceJob:', self.id, self.description,
-                    'done, calling callback', safe_encode=True)
+                    'done, calling callback')
 
         try:
             self.callback_on_done(self)
@@ -74,7 +74,7 @@ class DeviceJob(BaseJob):  # {{{
             pass
         if DEBUG:
             prints('DeviceJob:', self.id, self.description,
-                    'callback returned', safe_encode=True)
+                    'callback returned')
         self.job_manager.changed_queue.put(self)
 
     def report_progress(self, percent, msg=''):
@@ -105,14 +105,14 @@ class DeviceJob(BaseJob):  # {{{
             call_job_done = True
         self._aborted = True
         self.failed = True
-        self._details = unicode(err)
+        self._details = str(err)
         self.exception = err
         if call_job_done:
             self.job_done()
 
     @property
     def log_file(self):
-        return cStringIO.StringIO(self._details.encode('utf-8'))
+        return io.BytesIO(self._details.encode('utf-8'))
 
     # }}}
 
@@ -123,10 +123,10 @@ def device_name_for_plugboards(device_class):
     return device_class.__class__.__name__
 
 
-class BusyCursor(object):
+class BusyCursor:
 
     def __enter__(self):
-        QApplication.setOverrideCursor(QCursor(Qt.WaitCursor))
+        QApplication.setOverrideCursor(QCursor(Qt.CursorShape.WaitCursor))
 
     def __exit__(self, *args):
         QApplication.restoreOverrideCursor()
@@ -150,19 +150,19 @@ class DeviceManager(Thread):  # {{{
         self.unmanaged_devices = [x for x in self.devices if
                 x.MANAGES_DEVICE_PRESENCE]
         self.sleep_time     = sleep_time
-        self.connected_slot = connected_slot
+        self.connected_slot = connected_slot  # see DeviceMixin.device_connected()
         self.allow_connect_slot = allow_connect_slot
-        self.jobs           = Queue.Queue(0)
-        self.job_steps      = Queue.Queue(0)
+        self.jobs           = queue.Queue(0)
+        self.job_steps      = queue.Queue(0)
         self.keep_going     = True
         self.job_manager    = job_manager
-        self.reported_errors = set([])
+        self.reported_errors = set()
         self.current_job    = None
         self.scanner        = DeviceScanner()
         self.connected_device = None
         self.connected_device_kind = None
-        self.ejected_devices  = set([])
-        self.mount_connection_requests = Queue.Queue(0)
+        self.ejected_devices  = set()
+        self.mount_connection_requests = queue.Queue(0)
         self.open_feedback_slot = open_feedback_slot
         self.open_feedback_only_once_seen = set()
         self.after_callback_feedback_slot = after_callback_feedback_slot
@@ -240,7 +240,7 @@ class DeviceManager(Thread):  # {{{
             try:
                 job = self.jobs.get_nowait()
                 job.abort(Exception(_('Device no longer connected.')))
-            except Queue.Empty:
+            except queue.Empty:
                 break
         try:
             self.connected_device.post_yank_cleanup()
@@ -248,8 +248,9 @@ class DeviceManager(Thread):  # {{{
             pass
         if self.connected_device in self.ejected_devices:
             self.ejected_devices.remove(self.connected_device)
+            call_connected_slot = False
         else:
-            self.connected_slot(False, self.connected_device_kind)
+            call_connected_slot = True
         if self.call_shutdown_on_disconnect:
             # The current device is an instance of a plugin class instantiated
             # to handle this connection, probably as a mounted device. We are
@@ -261,6 +262,8 @@ class DeviceManager(Thread):  # {{{
         device_prefs.set_overrides()
         self.connected_device = currently_connected_device._device = None
         self._device_information = None
+        if call_connected_slot:
+            self.connected_slot(False, None)
 
     def detect_device(self):
         self.scanner.scan()
@@ -347,23 +350,23 @@ class DeviceManager(Thread):  # {{{
                 self.connected_device.eject()
                 if self.connected_device_kind != 'unmanaged-device':
                     self.ejected_devices.add(self.connected_device)
-                self.connected_slot(False, self.connected_device_kind)
+                self.connected_slot(False, None)
             elif hasattr(self.connected_device, 'unmount_device'):
                 # As we are on the wrong thread, this call must *not* do
                 # anything besides set a flag that the right thread will see.
                 self.connected_device.unmount_device()
 
-    def next(self):
+    def next_job(self):
         if not self.job_steps.empty():
             try:
                 return self.job_steps.get_nowait()
-            except Queue.Empty:
+            except queue.Empty:
                 pass
 
         if not self.jobs.empty():
             try:
                 return self.jobs.get_nowait()
-            except Queue.Empty:
+            except queue.Empty:
                 pass
 
     def run_startup(self, dev):
@@ -390,7 +393,7 @@ class DeviceManager(Thread):  # {{{
                 try:
                     (kls,device_kind, folder_path) = \
                                 self.mount_connection_requests.get_nowait()
-                except Queue.Empty:
+                except queue.Empty:
                     break
             if kls is not None:
                 try:
@@ -409,7 +412,7 @@ class DeviceManager(Thread):  # {{{
 
             do_sleep = True
             while True:
-                job = self.next()
+                job = self.next_job()
                 if job is not None:
                     do_sleep = False
                     self.current_job = job
@@ -470,7 +473,7 @@ class DeviceManager(Thread):  # {{{
         info = self.device.get_device_information(end_session=False)
         if len(info) < 5:
             info = tuple(list(info) + [{}])
-        info = [i.replace('\x00', '').replace('\x01', '') if isinstance(i, basestring) else i
+        info = [i.replace('\x00', '').replace('\x01', '') if isinstance(i, string_or_unicode) else i
                  for i in info]
         cp = self.device.card_prefix(end_session=False)
         fs = self.device.free_space()
@@ -503,7 +506,7 @@ class DeviceManager(Thread):  # {{{
             self._device_information['info'] = tuple(info)
 
     def get_current_device_information(self):
-        return self._device_information
+        return self._device_information if self.is_device_present else None
 
     def _books(self):
         '''Get metadata from device'''
@@ -561,7 +564,7 @@ class DeviceManager(Thread):  # {{{
             self.connected_device.set_plugboards(plugboards, find_plugboard)
         if metadata and files and len(metadata) == len(files):
             for f, mi in zip(files, metadata):
-                if isinstance(f, unicode):
+                if isinstance(f, str):
                     ext = f.rpartition('.')[-1].lower()
                     cpb = find_plugboard(
                             device_name_for_plugboards(self.connected_device),
@@ -602,7 +605,7 @@ class DeviceManager(Thread):  # {{{
                      metadata=None, plugboards=None, add_as_step_to_job=None):
         desc = ngettext('Upload one book to the device', 'Upload {} books to the device', len(names)).format(len(names))
         if titles:
-            desc += u': ' + u', '.join(titles)
+            desc += ': ' + ', '.join(titles)
         return self.create_job_step(self._upload_books, done, to_job=add_as_step_to_job,
                                args=[files, names],
                 kwargs={'on_card':on_card,'metadata':metadata,'plugboards':plugboards}, description=desc)
@@ -625,7 +628,7 @@ class DeviceManager(Thread):  # {{{
     def _save_books(self, paths, target):
         '''Copy books from device to disk'''
         for path in paths:
-            name = sanitize_file_name2(os.path.basename(path))
+            name = sanitize_file_name(os.path.basename(path))
             dest = os.path.join(target, name)
             if os.path.abspath(dest) != os.path.abspath(path):
                 with lopen(dest, 'wb') as f:
@@ -739,20 +742,20 @@ class DeviceMenu(QMenu):  # {{{
 
         delete_actions = [
                 ('main:', True, False,   I('reader.png'),
-                    _('Main Memory')),
+                    _('Main memory')),
                 ('carda:0', True, False,  I('sd.png'),
-                    _('Storage Card A')),
+                    _('Storage card A')),
                 ('cardb:0', True, False,  I('sd.png'),
-                    _('Storage Card B')),
+                    _('Storage card B')),
         ]
 
         specific_actions = [
                 ('main:', False, True,  I('reader.png'),
-                    _('Main Memory')),
+                    _('Main memory')),
                 ('carda:0', False, True, I('sd.png'),
-                    _('Storage Card A')),
+                    _('Storage card A')),
                 ('cardb:0', False, True, I('sd.png'),
-                    _('Storage Card B')),
+                    _('Storage card B')),
         ]
 
         later_menus = []
@@ -875,7 +878,9 @@ class DeviceSignals(QObject):  # {{{
 
     #: This signal is emitted once when the device is detected and once when
     #: it is disconnected. If the parameter is True, then it is a connection,
-    #: otherwise a disconnection.
+    #: otherwise a disconnection. Device information is not available in either
+    # case. If you need device information when connecting then use the
+    # device_metadata_available signal.
     device_connection_changed = pyqtSignal(object)
 
 
@@ -883,7 +888,7 @@ device_signals = DeviceSignals()
 # }}}
 
 
-class DeviceMixin(object):  # {{{
+class DeviceMixin:  # {{{
 
     def __init__(self, *args, **kwargs):
         pass
@@ -891,7 +896,7 @@ class DeviceMixin(object):  # {{{
     def init_device_mixin(self):
         self.device_error_dialog = error_dialog(self, _('Error'),
                 _('Error communicating with device'), ' ')
-        self.device_error_dialog.setModal(Qt.NonModal)
+        self.device_error_dialog.setModal(Qt.WindowModality.NonModal)
         self.device_manager = DeviceManager(FunctionDispatcher(self.device_detected),
                 self.job_manager, Dispatcher(self.status_bar.show_message),
                 Dispatcher(self.show_open_feedback),
@@ -928,7 +933,7 @@ class DeviceMixin(object):  # {{{
         d.show()
 
     def auto_convert_question(self, msg, autos):
-        autos = u'\n'.join(map(unicode, map(force_unicode, autos)))
+        autos = '\n'.join(map(str, map(force_unicode, autos)))
         return self.ask_a_yes_no_question(
                 _('No suitable formats'), msg,
                 ans_when_user_unavailable=True,
@@ -971,7 +976,7 @@ class DeviceMixin(object):  # {{{
         config_dialog.setWindowIcon(QIcon(I('config.png')))
         l = QVBoxLayout(config_dialog)
         config_dialog.setLayout(l)
-        bb = QDialogButtonBox(QDialogButtonBox.Ok|QDialogButtonBox.Cancel)
+        bb = QDialogButtonBox(QDialogButtonBox.StandardButton.Ok|QDialogButtonBox.StandardButton.Cancel)
         bb.accepted.connect(config_dialog.accept)
         bb.rejected.connect(config_dialog.reject)
         l.addWidget(cw)
@@ -984,7 +989,7 @@ class DeviceMixin(object):  # {{{
             if cw.validate():
                 QDialog.accept(config_dialog)
         config_dialog.accept = validate
-        if config_dialog.exec_() == config_dialog.Accepted:
+        if config_dialog.exec_() == QDialog.DialogCode.Accepted:
             dev.save_settings(cw)
 
             do_restart = show_restart_warning(_('Restart calibre for the changes to %s'
@@ -1026,7 +1031,7 @@ class DeviceMixin(object):  # {{{
 
         try:
             if 'Could not read 32 bytes on the control bus.' in \
-                    unicode(job.details):
+                    str(job.details):
                 error_dialog(self, _('Error talking to device'),
                              _('There was a temporary error talking to the '
                              'device. Please unplug and reconnect the device '
@@ -1067,6 +1072,8 @@ class DeviceMixin(object):  # {{{
     def device_detected(self, connected, device_kind):
         '''
         Called when a device is connected to the computer.
+
+        If connected is False then device_kind is None.
         '''
         # This can happen as this function is called in a queued connection and
         # the user could have yanked the device in the meantime
@@ -1098,7 +1105,12 @@ class DeviceMixin(object):  # {{{
             # Empty any device view information
             for v in dviews:
                 v.set_database([])
-            self.refresh_ondevice()
+            # Use a singleShot timer to ensure that the job event queue has
+            # emptied before the ondevice column is removed from the booklist.
+            # This deals with race conditions when repainting the booklist
+            # causing incorrect evaluation of the connected_device_name
+            # formatter function
+            QTimer.singleShot(0, self.refresh_ondevice)
         device_signals.device_connection_changed.emit(connected)
 
     def info_read(self, job):
@@ -1252,7 +1264,7 @@ class DeviceMixin(object):  # {{{
                 elif f in aval_out_formats:
                     formats.append((f, _('0 of %i books') % len(rows), True))
             d = ChooseFormatDeviceDialog(self, _('Choose format to send to device'), formats)
-            if d.exec_() != QDialog.Accepted:
+            if d.exec_() != QDialog.DialogCode.Accepted:
                 return
             if d.format():
                 fmt = d.format().lower()
@@ -1312,7 +1324,7 @@ class DeviceMixin(object):  # {{{
     def sync_catalogs(self, send_ids=None, do_auto_convert=True):
         if self.device_connected:
             settings = self.device_manager.device.settings()
-            ids = list(dynamic.get('catalogs_to_be_synced', set([]))) if send_ids is None else send_ids
+            ids = list(dynamic.get('catalogs_to_be_synced', set())) if send_ids is None else send_ids
             ids = [id for id in ids if self.library_view.model().db.has_id(id)]
             with BusyCursor():
                 files, _auto_ids = self.library_view.model().get_preferred_formats_from_ids(
@@ -1341,19 +1353,19 @@ class DeviceMixin(object):  # {{{
                         self.iactions['Convert Books'].auto_convert_catalogs(auto, format)
             files = [f for f in files if f is not None]
             if not files:
-                dynamic.set('catalogs_to_be_synced', set([]))
+                dynamic.set('catalogs_to_be_synced', set())
                 return
             metadata = self.library_view.model().metadata_for(ids)
             names = []
-            for mi in metadata:
+            for book_id, mi in zip(ids, metadata):
                 prefix = ascii_filename(mi.title)
-                if not isinstance(prefix, unicode):
+                if not isinstance(prefix, str):
                     prefix = prefix.decode(preferred_encoding, 'replace')
                 prefix = ascii_filename(prefix)
-                names.append('%s_%d%s'%(prefix, id,
-                    os.path.splitext(f)[1]))
+                names.append('%s_%d%s'%(prefix, book_id,
+                    os.path.splitext(files[-1])[1]))
                 self.update_thumbnail(mi)
-            dynamic.set('catalogs_to_be_synced', set([]))
+            dynamic.set('catalogs_to_be_synced', set())
             if files:
                 remove = []
                 space = {self.location_manager.free[0] : None,
@@ -1365,49 +1377,46 @@ class DeviceMixin(object):  # {{{
                         memory=[files, remove])
                 self.status_bar.show_message(_('Sending catalogs to device.'), 5000)
 
-    @dynamic_property
+    @property
     def news_to_be_synced(self):
-        doc = 'Set of ids to be sent to device'
+        'Set of ids to be sent to device'
+        ans = []
+        try:
+            ans = self.library_view.model().db.new_api.pref('news_to_be_synced',
+                    [])
+        except:
+            import traceback
+            traceback.print_exc()
+        return set(ans)
 
-        def fget(self):
-            ans = []
-            try:
-                ans = self.library_view.model().db.prefs.get('news_to_be_synced',
-                        [])
-            except:
-                import traceback
-                traceback.print_exc()
-            return set(ans)
-
-        def fset(self, ids):
-            try:
-                self.library_view.model().db.new_api.set_pref('news_to_be_synced',
-                        list(ids))
-            except:
-                import traceback
-                traceback.print_exc()
-
-        return property(fget=fget, fset=fset, doc=doc)
+    @news_to_be_synced.setter
+    def news_to_be_synced(self, ids):
+        try:
+            self.library_view.model().db.new_api.set_pref('news_to_be_synced',
+                    list(ids))
+        except:
+            import traceback
+            traceback.print_exc()
 
     def sync_news(self, send_ids=None, do_auto_convert=True):
         if self.device_connected:
             del_on_upload = config['delete_news_from_library_on_upload']
             settings = self.device_manager.device.settings()
             ids = list(self.news_to_be_synced) if send_ids is None else send_ids
-            ids = [id for id in ids if self.library_view.model().db.has_id(id)]
+            ids = [book_id for book_id in ids if self.library_view.model().db.has_id(book_id)]
             with BusyCursor():
                 files, _auto_ids = self.library_view.model().get_preferred_formats_from_ids(
                                 ids, settings.format_map,
                                 exclude_auto=do_auto_convert)
             auto = []
             if do_auto_convert and _auto_ids:
-                for id in _auto_ids:
-                    dbfmts = self.library_view.model().db.formats(id, index_is_id=True)
+                for book_id in _auto_ids:
+                    dbfmts = self.library_view.model().db.formats(book_id, index_is_id=True)
                     formats = [] if dbfmts is None else \
                         [f.lower() for f in dbfmts.split(',')]
                     if set(formats).intersection(available_input_formats()) \
                             and set(settings.format_map).intersection(available_output_formats()):
-                        auto.append(id)
+                        auto.append(book_id)
             if auto:
                 format = None
                 for fmt in settings.format_map:
@@ -1415,26 +1424,26 @@ class DeviceMixin(object):  # {{{
                         format = fmt
                         break
                 if format is not None:
-                    autos = [self.library_view.model().db.title(id, index_is_id=True) for id in auto]
+                    autos = [self.library_view.model().db.title(book_id, index_is_id=True) for book_id in auto]
                     if self.auto_convert_question(
                         _('Auto convert the following books before uploading to '
                             'the device?'), autos):
                         self.iactions['Convert Books'].auto_convert_news(auto, format)
             files = [f for f in files if f is not None]
             if not files:
-                self.news_to_be_synced = set([])
+                self.news_to_be_synced = set()
                 return
             metadata = self.library_view.model().metadata_for(ids)
             names = []
-            for mi in metadata:
+            for book_id, mi in zip(ids, metadata):
                 prefix = ascii_filename(mi.title)
-                if not isinstance(prefix, unicode):
+                if not isinstance(prefix, str):
                     prefix = prefix.decode(preferred_encoding, 'replace')
                 prefix = ascii_filename(prefix)
-                names.append('%s_%d%s'%(prefix, id,
-                    os.path.splitext(f)[1]))
+                names.append('%s_%d%s'%(prefix, book_id,
+                    os.path.splitext(files[-1])[1]))
                 self.update_thumbnail(mi)
-            self.news_to_be_synced = set([])
+            self.news_to_be_synced = set()
             if config['upload_news_to_device'] and files:
                 remove = ids if del_on_upload else []
                 space = {self.location_manager.free[0] : None,
@@ -1442,7 +1451,7 @@ class DeviceMixin(object):  # {{{
                     self.location_manager.free[2] : 'cardb'}
                 on_card = space.get(sorted(space.keys(), reverse=True)[0], None)
                 try:
-                    total_size = sum([os.stat(f).st_size for f in files])
+                    total_size = sum(os.stat(f).st_size for f in files)
                 except:
                     try:
                         import traceback
@@ -1492,8 +1501,8 @@ class DeviceMixin(object):  # {{{
 
         bad, good, gf, names, remove_ids = [], [], [], [], []
         for f in _files:
-            mi = imetadata.next()
-            id = ids.next()
+            mi = next(imetadata)
+            id = next(ids)
             if f is None:
                 bad.append(mi.title)
             else:
@@ -1507,7 +1516,7 @@ class DeviceMixin(object):  # {{{
                 if not a:
                     a = _('Unknown')
                 prefix = ascii_filename(t+' - '+a)
-                if not isinstance(prefix, unicode):
+                if not isinstance(prefix, str):
                     prefix = prefix.decode(preferred_encoding, 'replace')
                 prefix = ascii_filename(prefix)
                 names.append('%s_%d%s'%(prefix, id, os.path.splitext(f)[1]))
@@ -1564,7 +1573,7 @@ class DeviceMixin(object):  # {{{
         '''
         Upload metadata to device.
         '''
-        plugboards = self.library_view.model().db.prefs.get('plugboards', {})
+        plugboards = self.library_view.model().db.new_api.pref('plugboards', {})
         self.device_manager.sync_booklists(Dispatcher(lambda x: x),
                                            self.booklists(), plugboards)
 
@@ -1572,7 +1581,7 @@ class DeviceMixin(object):  # {{{
         '''
         Upload metadata to device.
         '''
-        plugboards = self.library_view.model().db.prefs.get('plugboards', {})
+        plugboards = self.library_view.model().db.new_api.pref('plugboards', {})
         self.device_manager.sync_booklists(FunctionDispatcher(self.metadata_synced),
                                            self.booklists(), plugboards,
                                            add_as_step_to_job=add_as_step_to_job)
@@ -1612,7 +1621,7 @@ class DeviceMixin(object):  # {{{
         :param files: List of either paths to files or file like objects
         '''
         titles = [i.title for i in metadata]
-        plugboards = self.library_view.model().db.prefs.get('plugboards', {})
+        plugboards = self.library_view.model().db.new_api.pref('plugboards', {})
         job = self.device_manager.upload_books(
                 FunctionDispatcher(self.books_uploaded),
                 files, names, on_card=on_card,
@@ -1639,7 +1648,7 @@ class DeviceMixin(object):  # {{{
                 d.exec_()
             elif isinstance(job.exception, WrongDestinationError):
                 error_dialog(self, _('Incorrect destination'),
-                        unicode(job.exception), show=True)
+                        str(job.exception), show=True)
             else:
                 self.device_job_exception(job)
             return
@@ -1703,7 +1712,7 @@ class DeviceMixin(object):  # {{{
         the memory locations. The fifth is a set of paths to the
         matching books on the device.
         '''
-        loc = [None, None, None, 0, set([])]
+        loc = [None, None, None, 0, set()]
 
         if reset:
             self.book_db_id_cache = None
@@ -1771,7 +1780,10 @@ class DeviceMixin(object):  # {{{
         string_pat = re.compile(r'(?u)\W|[_]')
 
         def clean_string(x):
-            x = x.lower() if x else ''
+            try:
+                x = x.lower() if x else ''
+            except Exception:
+                x = ''
             return string_pat.sub('', x)
 
         update_metadata = (
@@ -1884,7 +1896,7 @@ class DeviceMixin(object):  # {{{
                     # loop with preventing App Not Responding errors
                     if current_book_count % 10 == 0:
                         QCoreApplication.processEvents(
-                            flags=QEventLoop.ExcludeUserInputEvents|QEventLoop.ExcludeSocketNotifiers)
+                            flags=QEventLoop.ProcessEventsFlag.ExcludeUserInputEvents|QEventLoop.ProcessEventsFlag.ExcludeSocketNotifiers)
                     current_book_count += 1
                     book.in_library = None
                     if getattr(book, 'uuid', None) in self.db_book_uuid_cache:
@@ -1945,7 +1957,7 @@ class DeviceMixin(object):  # {{{
 
             if update_metadata:
                 if self.device_manager.is_device_connected:
-                    plugboards = self.library_view.model().db.prefs.get('plugboards', {})
+                    plugboards = self.library_view.model().db.new_api.pref('plugboards', {})
                     self.device_manager.sync_booklists(
                                 FunctionDispatcher(self.metadata_synced), booklists,
                                 plugboards, add_as_step_to_job)
