@@ -8,14 +8,13 @@ __docformat__ = 'restructuredtext en'
 
 from collections import namedtuple
 
-from PyQt5.Qt import (
-    QWidget, Qt, QLabel, QVBoxLayout, QDialogButtonBox, QApplication, QTimer, QPixmap,
+from qt.core import (
+    QWidget, Qt, QLabel, QVBoxLayout, QDialogButtonBox, QApplication, QTimer, QPixmap, QEvent,
     QSize, pyqtSignal, QIcon, QPlainTextEdit, QCheckBox, QPainter, QHBoxLayout, QFontMetrics,
     QPainterPath, QRectF, pyqtProperty, QPropertyAnimation, QEasingCurve, QSizePolicy, QImage, QPalette)
 
 from calibre.constants import __version__
 from calibre.gui2.dialogs.message_box import ViewLog
-from polyglot.builtins import unicode_type
 
 Question = namedtuple('Question', 'payload callback cancel_callback '
         'title msg html_log log_viewer_title log_is_file det_msg '
@@ -114,17 +113,17 @@ class ProceedQuestion(QWidget):
         self.bb = QDialogButtonBox()
         self.bb.accepted.connect(self.accept)
         self.bb.rejected.connect(self.reject)
-        self.log_button = self.bb.addButton(_('View log'), self.bb.ActionRole)
+        self.log_button = self.bb.addButton(_('View log'), QDialogButtonBox.ButtonRole.ActionRole)
         self.log_button.setIcon(QIcon(I('debug.png')))
         self.log_button.clicked.connect(self.show_log)
         self.copy_button = self.bb.addButton(_('&Copy to clipboard'),
-                self.bb.ActionRole)
+                QDialogButtonBox.ButtonRole.ActionRole)
         self.copy_button.clicked.connect(self.copy_to_clipboard)
-        self.action_button = self.bb.addButton('', self.bb.ActionRole)
+        self.action_button = self.bb.addButton('', QDialogButtonBox.ButtonRole.ActionRole)
         self.action_button.clicked.connect(self.action_clicked)
         self.show_det_msg = _('Show &details')
         self.hide_det_msg = _('Hide &details')
-        self.det_msg_toggle = self.bb.addButton(self.show_det_msg, self.bb.ActionRole)
+        self.det_msg_toggle = self.bb.addButton(self.show_det_msg, QDialogButtonBox.ButtonRole.ActionRole)
         self.det_msg_toggle.clicked.connect(self.toggle_det_msg)
         self.det_msg_toggle.setToolTip(
                 _('Show detailed information about this error'))
@@ -160,7 +159,7 @@ class ProceedQuestion(QWidget):
         t.setSingleShot(True), t.setInterval(100), t.timeout.connect(self.parent_resized)
 
     def eventFilter(self, obj, ev):
-        if ev.type() == ev.Resize and self.isVisible():
+        if ev.type() == QEvent.Type.Resize and self.isVisible():
             self.resize_timer.start()
         return False
 
@@ -171,9 +170,9 @@ class ProceedQuestion(QWidget):
     def copy_to_clipboard(self, *args):
         QApplication.clipboard().setText(
                 'calibre, version %s\n%s: %s\n\n%s' %
-                (__version__, unicode_type(self.windowTitle()),
-                    unicode_type(self.msg_label.text()),
-                    unicode_type(self.det_msg.toPlainText())))
+                (__version__, str(self.windowTitle()),
+                    str(self.msg_label.text()),
+                    str(self.det_msg.toPlainText())))
         self.copy_button.setText(_('Copied'))
 
     def action_clicked(self):
@@ -211,7 +210,7 @@ class ProceedQuestion(QWidget):
         self.show_question()
 
     def toggle_det_msg(self, *args):
-        vis = unicode_type(self.det_msg_toggle.text()) == self.hide_det_msg
+        vis = str(self.det_msg_toggle.text()) == self.hide_det_msg
         self.det_msg_toggle.setText(self.show_det_msg if vis else
                 self.hide_det_msg)
         self.det_msg.setVisible(not vis)
@@ -406,7 +405,7 @@ class ProceedQuestion(QWidget):
 
 def main():
     from calibre.gui2 import Application
-    from PyQt5.Qt import QMainWindow, QStatusBar, QTimer
+    from qt.core import QMainWindow, QStatusBar, QTimer
     app = Application([])
     w = QMainWindow()
     s = QStatusBar(w)
