@@ -13,7 +13,7 @@ from qt.core import (
     Qt, QIcon, QWidget, QHBoxLayout, QVBoxLayout, QToolButton, QLabel, QFrame, QDialog, QComboBox, QLineEdit,
     QTimer, QMenu, QActionGroup, QAction, QSizePolicy, pyqtSignal)
 
-from calibre.gui2 import error_dialog, question_dialog, gprefs
+from calibre.gui2 import error_dialog, question_dialog, gprefs, config
 from calibre.gui2.widgets import HistoryLineEdit
 from calibre.library.field_metadata import category_icon_map
 from calibre.utils.icu import sort_key
@@ -22,10 +22,10 @@ from calibre.ebooks.metadata import title_sort
 from calibre.gui2.dialogs.tag_categories import TagCategories
 from calibre.gui2.dialogs.tag_list_editor import TagListEditor
 from calibre.gui2.dialogs.edit_authors_dialog import EditAuthorsDialog
-from polyglot.builtins import unicode_type, iteritems
+from polyglot.builtins import iteritems
 
 
-class TagBrowserMixin(object):  # {{{
+class TagBrowserMixin:  # {{{
 
     def __init__(self, *args, **kwargs):
         pass
@@ -125,7 +125,7 @@ class TagBrowserMixin(object):  # {{{
             if new_cat not in user_cats:
                 break
             i += 1
-            n = new_name + unicode_type(i)
+            n = new_name + str(i)
         # Add the new category
         user_cats[new_cat] = []
         db.new_api.set_pref('user_categories', user_cats)
@@ -136,6 +136,9 @@ class TagBrowserMixin(object):  # {{{
         self.tags_view.show_item_at_index(idx)
         # Open the editor on the new item to rename it
         if new_category_name is None:
+            item = m.get_node(idx)
+            item.use_vl = False
+            item.ignore_vl = True
             self.tags_view.edit(idx)
 
     def do_edit_user_categories(self, on_category=None):
@@ -279,7 +282,7 @@ class TagBrowserMixin(object):  # {{{
                     m.delete_item_from_all_user_categories(orig_name[item], category)
                 for old_id in to_rename:
                     m.rename_item_in_all_user_categories(orig_name[old_id],
-                                            category, unicode_type(to_rename[old_id]))
+                                            category, str(to_rename[old_id]))
 
                 db.new_api.remove_items(category, to_delete)
                 db.new_api.rename_items(category, to_rename, change_index=False)
@@ -553,7 +556,7 @@ class TagBrowserBar(QWidget):  # {{{
         ac = QAction(parent)
         parent.addAction(ac)
         parent.keyboard.register_shortcut('tag browser find box',
-                _('Find next match'), default_keys=(),
+                _('Find in the Tag browser'), default_keys=(),
                 action=ac, group=_('Tag browser'))
         ac.triggered.connect(self.set_focus_to_find_box)
 
@@ -565,7 +568,7 @@ class TagBrowserBar(QWidget):  # {{{
         ac = QAction(parent)
         parent.addAction(ac)
         parent.keyboard.register_shortcut('tag browser find button',
-                _('Find in the Tag browser'), default_keys=(),
+                _('Find next match'), default_keys=(),
                 action=ac, group=_('Tag browser'))
         ac.triggered.connect(self.search_button.click)
 
@@ -577,7 +580,7 @@ class TagBrowserBar(QWidget):  # {{{
         b.setIcon(QIcon(I('search.png')))
         b.setCheckable(True)
         b.setChecked(gprefs.get('tag browser search box visible', False))
-        b.setToolTip(_('Search for items in the Tag browser'))
+        b.setToolTip(_('Find item in the Tag browser'))
         b.setAutoRaise(True)
         b.toggled.connect(self.update_searchbar_state)
         self.update_searchbar_state()
@@ -662,8 +665,6 @@ class TagBrowserWidget(QFrame):  # {{{
         self.not_found_label_timer.setSingleShot(True)
         self.not_found_label_timer.timeout.connect(self.not_found_label_timer_event,
                                                    type=Qt.ConnectionType.QueuedConnection)
-        # The Alter Tag Browser button
-        l = self.alter_tb
         self.collapse_all_action = ac = QAction(parent)
         parent.addAction(ac)
         parent.keyboard.register_shortcut('tag browser collapse all',
@@ -671,6 +672,8 @@ class TagBrowserWidget(QFrame):  # {{{
                 action=ac, group=_('Tag browser'))
         connect_lambda(ac.triggered, self, lambda self: self.tags_view.collapseAll())
 
+        # The Configure Tag Browser button
+        l = self.alter_tb
         ac = QAction(parent)
         parent.addAction(ac)
         parent.keyboard.register_shortcut('tag browser alter',
@@ -678,6 +681,11 @@ class TagBrowserWidget(QFrame):  # {{{
                 action=ac, group=_('Tag browser'))
         ac.triggered.connect(l.showMenu)
 
+        l.m.aboutToShow.connect(self.about_to_show_configure_menu)
+        l.m.show_counts_action = ac = l.m.addAction('counts')
+        ac.triggered.connect(self.toggle_counts)
+        l.m.show_avg_rating_action = ac = l.m.addAction('avg rating')
+        ac.triggered.connect(self.toggle_avg_rating)
         sb = l.m.addAction(_('Sort by'))
         sb.m = l.sort_menu = QMenu(l.m)
         sb.setMenu(sb.m)
@@ -736,6 +744,18 @@ class TagBrowserWidget(QFrame):  # {{{
         # self.leak_test_timer.timeout.connect(self.test_for_leak)
         # self.leak_test_timer.start(5000)
 
+    def about_to_show_configure_menu(self):
+        ac = self.alter_tb.m.show_counts_action
+        ac.setText(_('Hide counts') if gprefs['tag_browser_show_counts'] else _('Show counts'))
+        ac = self.alter_tb.m.show_avg_rating_action
+        ac.setText(_('Hide average rating') if config['show_avg_rating'] else _('Show average rating'))
+
+    def toggle_counts(self):
+        gprefs['tag_browser_show_counts'] ^= True
+
+    def toggle_avg_rating(self):
+        config['show_avg_rating'] ^= True
+
     def save_state(self):
         gprefs.set('tag browser search box visible', self.toggle_search_button.isChecked())
 
@@ -771,7 +791,7 @@ class TagBrowserWidget(QFrame):  # {{{
 
     @property
     def find_text(self):
-        return unicode_type(self.item_search.currentText()).strip()
+        return str(self.item_search.currentText()).strip()
 
     def reset_find(self):
         model = self.tags_view.model()
