@@ -11,7 +11,7 @@ Logic for setting up conversion jobs
 
 import os
 
-from PyQt5.Qt import QDialog, QProgressDialog, QTimer
+from qt.core import QDialog, QProgressDialog, QTimer
 
 from calibre.ptempfile import PersistentTemporaryFile
 from calibre.gui2 import warning_dialog, question_dialog
@@ -24,7 +24,7 @@ from calibre.ebooks.conversion.config import (
         GuiRecommendations, load_defaults, load_specifics, save_specifics,
         get_input_format_for_book, NoSupportedInputFormats)
 from calibre.gui2.convert import bulk_defaults_for_input_format
-from polyglot.builtins import unicode_type, as_bytes
+from polyglot.builtins import as_bytes
 
 
 def convert_single_ebook(parent, db, book_ids, auto_conversion=False,  # {{{
@@ -45,11 +45,11 @@ def convert_single_ebook(parent, db, book_ids, auto_conversion=False,  # {{{
 
             if auto_conversion:
                 d.accept()
-                result = QDialog.Accepted
+                result = QDialog.DialogCode.Accepted
             else:
                 result = d.exec_()
 
-            if result == QDialog.Accepted:
+            if result == QDialog.DialogCode.Accepted:
                 # if not convert_existing(parent, db, [book_id], d.output_format):
                 #    continue
 
@@ -67,7 +67,7 @@ def convert_single_ebook(parent, db, book_ids, auto_conversion=False,  # {{{
                 temp_files = [in_file]
 
                 try:
-                    dtitle = unicode_type(mi.title)
+                    dtitle = str(mi.title)
                 except:
                     dtitle = repr(mi.title)
                 desc = _('Convert book %(num)d of %(total)d (%(title)s)') % \
@@ -143,7 +143,7 @@ def convert_bulk_ebook(parent, queue, db, book_ids, out_format=None, args=[]):
 
     d = BulkConfig(parent, db, out_format,
             has_saved_settings=has_saved_settings, book_ids=book_ids)
-    if d.exec_() != QDialog.Accepted:
+    if d.exec_() != QDialog.DialogCode.Accepted:
         return None
 
     output_format = d.output_format
@@ -228,7 +228,7 @@ class QueueBulk(QProgressDialog):
                 if x[0] == 'debug_pipeline':
                     lrecs.remove(x)
             try:
-                dtitle = unicode_type(mi.title)
+                dtitle = str(mi.title)
             except:
                 dtitle = repr(mi.title)
             if len(dtitle) > 50:
@@ -327,7 +327,7 @@ def generate_catalog(parent, dbspec, ids, device_manager, db):  # {{{
     # Build the Catalog dialog in gui2.dialogs.catalog
     d = Catalog(parent, dbspec, ids, db)
 
-    if d.exec_() != d.Accepted:
+    if d.exec_() != QDialog.DialogCode.Accepted:
         return None
 
     # Create the output file
