@@ -6,7 +6,7 @@ __license__ = 'GPL v3'
 __copyright__ = '2013, Kovid Goyal <kovid at kovidgoyal.net>'
 
 import string
-from polyglot.builtins import iteritems, map
+from polyglot.builtins import iteritems
 
 from calibre.utils.config import JSONConfig
 from calibre.spell.dictionary import Dictionaries, parse_lang_code
@@ -56,6 +56,7 @@ d['remove_unused_classes'] = False
 d['merge_identical_selectors'] = False
 d['merge_identical_selectors'] = False
 d['merge_rules_with_identical_properties'] = False
+d['remove_unreferenced_sheets'] = True
 d['global_book_toolbar'] = [
 'new-file', 'open-book',  'save-book', None, 'global-undo', 'global-redo', 'create-checkpoint', None, 'donate', 'user-manual']
 d['global_tools_toolbar'] = [
