@@ -69,9 +69,7 @@ from calibre.utils.config import JSONConfig
 from calibre.utils.icu import numeric_sort_key
 from calibre.utils.imghdr import identify
 from calibre.utils.tdir_in_cache import tdir_in_cache
-from polyglot.builtins import (
-    as_bytes, iteritems, itervalues, map, string_or_bytes, unicode_type
-)
+from polyglot.builtins import as_bytes, iteritems, itervalues, string_or_bytes
 from polyglot.urllib import urlparse
 
 _diff_dialogs = []
@@ -390,7 +388,7 @@ class Boss(QObject):
                     import traceback
                     traceback.print_exc()
             if ef:
-                if isinstance(ef, unicode_type):
+                if isinstance(ef, str):
                     ef = [ef]
                 tuple(map(self.gui.file_list.request_edit, ef))
             else:
@@ -469,6 +467,8 @@ class Boss(QObject):
             self.gui.update_window_title()
 
     def reorder_spine(self, items):
+        if not self.ensure_book():
+            return
         self.add_savepoint(_('Before: Re-order text'))
         c = current_container()
         c.set_spine(items)
@@ -554,6 +554,8 @@ class Boss(QObject):
                 show_font_face_rule_for_font_files(c, added_fonts, self.gui)
 
     def add_cover(self):
+        if not self.ensure_book():
+            return
         d = AddCover(current_container(), self.gui)
         d.import_requested.connect(self.do_add_file)
         try:
@@ -567,7 +569,8 @@ class Boss(QObject):
         finally:
             d.import_requested.disconnect()
 
-    def ensure_book(self, msg):
+    def ensure_book(self, msg=None):
+        msg = msg or _('No book is currently open. You must first open a book.')
         if current_container() is None:
             error_dialog(self.gui, _('No book open'), msg, show=True)
             return False
@@ -588,6 +591,8 @@ class Boss(QObject):
             self.gui.file_list.build(current_container())
 
     def insert_inline_toc(self):
+        if not self.ensure_book():
+            return
         self.commit_all_editors_to_container()
         self.add_savepoint(_('Before: Insert inline Table of Contents'))
         name = create_inline_toc(current_container())
@@ -601,6 +606,8 @@ class Boss(QObject):
         self.edit_file(name, 'html')
 
     def polish(self, action, name, parent=None):
+        if not self.ensure_book():
+            return
         from calibre.gui2.tweak_book.polish import get_customization, show_report
         customization = get_customization(action, name, parent or self.gui)
         if customization is None:
@@ -667,6 +674,8 @@ class Boss(QObject):
             self.rewind_savepoint()
 
     def manage_fonts(self):
+        if not self.ensure_book(_('No book is currently open. You must first open a book to manage fonts.')):
+            return
         self.commit_all_editors_to_container()
         self.gui.manage_fonts.display()
 
@@ -680,6 +689,8 @@ class Boss(QObject):
     # Renaming {{{
 
     def rationalize_folders(self):
+        if not self.ensure_book():
+            return
         c = current_container()
         if not c.SUPPORTS_FILENAMES:
             return error_dialog(self.gui, _('Not supported'),
@@ -735,7 +746,7 @@ class Boss(QObject):
     def rename_done(self, name_map, job, from_filelist=None):
         if job.traceback is not None:
             return error_dialog(self.gui, _('Failed to rename files'),
-                    _('Failed to rename files, click Show details for more information.'),
+                    _('Failed to rename files, click "Show details" for more information.'),
                                 det_msg=job.traceback, show=True)
         self.gui.file_list.build(current_container())
         self.set_modified()
@@ -846,6 +857,8 @@ class Boss(QObject):
             self.show_current_diff(allow_revert=allow_revert, to_container=to_container)
 
     def compare_book(self):
+        if not self.ensure_book():
+            return
         self.commit_all_editors_to_container()
         c = current_container()
         path = choose_files(self.gui, 'select-book-for-comparison', _('Choose book'), filters=[
@@ -973,13 +986,15 @@ class Boss(QObject):
                     'No class {} found to change').format(class_name), show=True)
 
     def set_semantics(self):
+        if not self.ensure_book():
+            return
         self.commit_all_editors_to_container()
         c = current_container()
         if c.book_type == 'azw3':
             return error_dialog(self.gui, _('Not supported'), _(
                 'Semantics are not supported for the AZW3 format.'), show=True)
         d = InsertSemantics(c, parent=self.gui)
-        if d.exec_() == QDialog.DialogCode.Accepted and d.changed_type_map:
+        if d.exec_() == QDialog.DialogCode.Accepted and d.changes:
             self.add_savepoint(_('Before: Set Semantics'))
             d.apply_changes(current_container())
             self.apply_container_update_to_gui()
@@ -1261,7 +1276,7 @@ class Boss(QObject):
     def copy_saved(self, job):
         if job.traceback is not None:
             return error_dialog(self.gui, _('Failed to save copy'),
-                    _('Failed to save copy, click Show details for more information.'), det_msg=job.traceback, show=True)
+                    _('Failed to save copy, click "Show details" for more information.'), det_msg=job.traceback, show=True)
         msg = _('Copy saved to %s') % job.result
         info_dialog(self.gui, _('Copy saved'), msg, show=True)
         self.gui.show_status_message(msg, 5)
@@ -1428,7 +1443,7 @@ class Boss(QObject):
             if '/' in name or os.sep in name:
                 try:
                     os.makedirs(os.path.dirname(dest))
-                except EnvironmentError as err:
+                except OSError as err:
                     if err.errno != errno.EEXIST:
                         raise
             self.export_file(name, dest)
