@@ -5,15 +5,17 @@
 __license__ = 'GPL v3'
 __copyright__ = '2014, Kovid Goyal <kovid at kovidgoyal.net>'
 
-import re, copy, weakref
+import copy
+import re
+import weakref
 from collections import OrderedDict, namedtuple
 from itertools import groupby
 from operator import attrgetter, itemgetter
-
 from qt.core import (
-    Qt, QObject, QSize, QVBoxLayout, QStackedLayout, QWidget, QLineEdit, QListView,
-    QToolButton, QIcon, QHBoxLayout, QPushButton, QListWidget, QListWidgetItem,
-    QGridLayout, QPlainTextEdit, QLabel, QFrame, QDialog, QDialogButtonBox, QTextCursor)
+    QDialog, QDialogButtonBox, QFrame, QGridLayout, QHBoxLayout, QIcon, QLabel,
+    QLineEdit, QListView, QListWidget, QListWidgetItem, QObject, QPushButton, QSize,
+    QStackedLayout, Qt, QTextCursor, QToolButton, QVBoxLayout, QWidget
+)
 
 from calibre.constants import ismacos
 from calibre.gui2 import error_dialog
@@ -23,9 +25,9 @@ from calibre.gui2.tweak_book.widgets import Dialog, PlainTextEdit
 from calibre.utils.config import JSONConfig
 from calibre.utils.icu import string_length as strlen
 from calibre.utils.localization import localize_user_manual_link
-from polyglot.builtins import codepoint_to_chr, iteritems, itervalues, unicode_type, range
+from polyglot.builtins import codepoint_to_chr, iteritems, itervalues
 
-string_length = lambda x: strlen(unicode_type(x))  # Needed on narrow python builds, as subclasses of unicode dont work
+string_length = lambda x: strlen(str(x))  # Needed on narrow python builds, as subclasses of unicode dont work
 KEY = Qt.Key.Key_J
 MODIFIER = Qt.Modifier.META if ismacos else Qt.Modifier.CTRL
 
@@ -103,7 +105,7 @@ def escape_funcs():
     return escape, unescape
 
 
-class TabStop(unicode_type):
+class TabStop(str):
 
     def __new__(self, raw, start_offset, tab_stops, is_toplevel=True):
         if raw.endswith('}'):
@@ -114,7 +116,7 @@ class TabStop(unicode_type):
             for c in child_stops:
                 c.parent = self
             tab_stops.extend(child_stops)
-            self = unicode_type.__new__(self, uraw)
+            self = str.__new__(self, uraw)
             if num.endswith('*'):
                 self.takes_selection = True
                 num = num[:-1]
@@ -122,7 +124,7 @@ class TabStop(unicode_type):
                 self.takes_selection = False
             self.num = int(num)
         else:
-            self = unicode_type.__new__(self, '')
+            self = str.__new__(self, '')
             self.num = int(raw[1:])
             self.takes_selection = False
         self.start = start_offset
@@ -134,7 +136,7 @@ class TabStop(unicode_type):
 
     def __repr__(self):
         return 'TabStop(text=%s num=%d start=%d is_mirror=%s takes_selection=%s is_toplevel=%s)' % (
-            unicode_type.__repr__(self), self.num, self.start, self.is_mirror, self.takes_selection, self.is_toplevel)
+            str.__repr__(self), self.num, self.start, self.is_mirror, self.takes_selection, self.is_toplevel)
 
 
 def parse_template(template, start_offset=0, is_toplevel=True, grouped=True):
@@ -173,7 +175,7 @@ def snippets(refresh=False):
     if _snippets is None or refresh:
         _snippets = copy.deepcopy(builtin_snippets)
         for snip in user_snippets.get('snippets', []):
-            if snip['trigger'] and isinstance(snip['trigger'], unicode_type):
+            if snip['trigger'] and isinstance(snip['trigger'], str):
                 key = snip_key(snip['trigger'], *snip['syntaxes'])
                 _snippets[key] = {'template':snip['template'], 'description':snip['description']}
         _snippets = sorted(iteritems(_snippets), key=(lambda key_snip:string_length(key_snip[0].trigger)), reverse=True)
@@ -182,7 +184,7 @@ def snippets(refresh=False):
 # Editor integration {{{
 
 
-class EditorTabStop(object):
+class EditorTabStop:
 
     def __init__(self, left, tab_stops, editor):
         self.editor = weakref.ref(editor)
@@ -481,7 +483,7 @@ class EditSnippet(QWidget):
         t.setPlaceholderText(_('The text used to trigger this snippet'))
         add_row(_('Tri&gger:'), t)
 
-        self.template = t = QPlainTextEdit(self)
+        self.template = t = PlainTextEdit(self)
         la.setBuddy(t)
         add_row(_('&Template:'), t)
 
