@@ -9,7 +9,7 @@ import re
 import regex
 import textwrap
 import unicodedata
-from PyQt5.Qt import (
+from qt.core import (
     QColor, QColorDialog, QFont, QFontDatabase, QKeySequence, QPainter, QPalette,
     QPlainTextEdit, QRect, QSize, Qt, QTextCursor, QTextEdit, QTextFormat, QTimer,
     QToolTip, QWidget, pyqtSignal
@@ -24,8 +24,8 @@ from calibre.gui2.tweak_book import (
 )
 from calibre.gui2.tweak_book.completion.popup import CompletionPopup
 from calibre.gui2.tweak_book.editor import (
-    LINK_PROPERTY, SPELL_LOCALE_PROPERTY, SPELL_PROPERTY, SYNTAX_PROPERTY,
-    store_locale
+    CLASS_ATTRIBUTE_PROPERTY, LINK_PROPERTY, SPELL_LOCALE_PROPERTY, SPELL_PROPERTY,
+    SYNTAX_PROPERTY, store_locale
 )
 from calibre.gui2.tweak_book.editor.smarts import NullSmarts
 from calibre.gui2.tweak_book.editor.snippets import SnippetManager
@@ -40,7 +40,7 @@ from calibre.utils.icu import (
 )
 from calibre.utils.img import image_to_data
 from calibre.utils.titlecase import titlecase
-from polyglot.builtins import as_unicode, map, range, unicode_type
+from polyglot.builtins import as_unicode
 
 
 def get_highlighter(syntax):
@@ -67,7 +67,7 @@ _dff = None
 def default_font_family():
     global _dff
     if _dff is None:
-        families = set(map(unicode_type, QFontDatabase().families()))
+        families = set(map(str, QFontDatabase().families()))
         for x in ('Ubuntu Mono', 'Consolas', 'Liberation Mono'):
             if x in families:
                 _dff = x
@@ -93,6 +93,7 @@ class LineNumbers(QWidget):  # {{{
 class TextEdit(PlainTextEdit):
 
     link_clicked = pyqtSignal(object)
+    class_clicked = pyqtSignal(object)
     smart_highlighting_updated = pyqtSignal()
 
     def __init__(self, parent=None, expected_geometry=(100, 50)):
@@ -259,7 +260,7 @@ class TextEdit(PlainTextEdit):
         self.setFont(font)
         self.highlighter.apply_theme(theme)
         w = self.fontMetrics()
-        self.number_width = max(map(lambda x:w.width(unicode_type(x)), range(10)))
+        self.number_width = max(map(lambda x:w.width(str(x)), range(10)))
         self.size_hint = QSize(self.expected_geometry[0] * w.averageCharWidth(), self.expected_geometry[1] * w.height())
         self.highlight_color = theme_color(theme, 'HighlightRegion', 'bg')
         self.highlight_cursor_line()
@@ -277,7 +278,9 @@ class TextEdit(PlainTextEdit):
             if self.smarts.override_tab_stop_width is not None:
                 self.tw = self.smarts.override_tab_stop_width
                 self.setTabStopWidth(self.tw * self.space_width)
-        self.setPlainText(unicodedata.normalize('NFC', unicode_type(text)))
+        if isinstance(text, bytes):
+            text = text.decode('utf-8', 'replace')
+        self.setPlainText(unicodedata.normalize('NFC', str(text)))
         if process_template and QPlainTextEdit.find(self, '%CURSOR%'):
             c = self.textCursor()
             c.insertText('')
@@ -311,7 +314,7 @@ class TextEdit(PlainTextEdit):
         c.movePosition(QTextCursor.MoveOperation.NextBlock, n=lnum - 1)
         c.movePosition(QTextCursor.MoveOperation.StartOfLine)
         c.movePosition(QTextCursor.MoveOperation.EndOfLine, QTextCursor.MoveMode.KeepAnchor)
-        text = unicode_type(c.selectedText()).rstrip('\0')
+        text = str(c.selectedText()).rstrip('\0')
         if col is None:
             c.movePosition(QTextCursor.MoveOperation.StartOfLine)
             lt = text.lstrip()
@@ -370,7 +373,7 @@ class TextEdit(PlainTextEdit):
         if wrap:
             pos = m_end if reverse else m_start
         c.setPosition(pos, QTextCursor.MoveMode.KeepAnchor)
-        raw = unicode_type(c.selectedText()).replace(PARAGRAPH_SEPARATOR, '\n').rstrip('\0')
+        raw = str(c.selectedText()).replace(PARAGRAPH_SEPARATOR, '\n').rstrip('\0')
         m = pat.search(raw)
         if m is None:
             return False
@@ -403,7 +406,7 @@ class TextEdit(PlainTextEdit):
         if self.current_search_mark is None:
             return 0
         c = self.current_search_mark.cursor
-        raw = unicode_type(c.selectedText()).replace(PARAGRAPH_SEPARATOR, '\n').rstrip('\0')
+        raw = str(c.selectedText()).replace(PARAGRAPH_SEPARATOR, '\n').rstrip('\0')
         if template is None:
             count = len(pat.findall(raw))
         else:
@@ -417,7 +420,7 @@ class TextEdit(PlainTextEdit):
                 if getattr(template.func, 'append_final_output_to_marked', False):
                     retval = template.end()
                     if retval:
-                        raw += unicode_type(retval)
+                        raw += str(retval)
                 else:
                     template.end()
                 show_function_debug_output(template)
@@ -440,7 +443,7 @@ class TextEdit(PlainTextEdit):
             c = self.textCursor()
             c.beginEditBlock()
             c.movePosition(QTextCursor.MoveOperation.Start), c.movePosition(QTextCursor.MoveOperation.End, QTextCursor.MoveMode.KeepAnchor)
-            text = unicode_type(c.selectedText()).replace(PARAGRAPH_SEPARATOR, '\n').rstrip('\0')
+            text = str(c.selectedText()).replace(PARAGRAPH_SEPARATOR, '\n').rstrip('\0')
             from calibre.ebooks.oeb.polish.css import sort_sheet
             text = css_text(sort_sheet(current_container(), text))
             c.insertText(text)
@@ -461,7 +464,7 @@ class TextEdit(PlainTextEdit):
         if wrap and not complete:
             pos = QTextCursor.MoveOperation.End if reverse else QTextCursor.MoveOperation.Start
         c.movePosition(pos, QTextCursor.MoveMode.KeepAnchor)
-        raw = unicode_type(c.selectedText()).replace(PARAGRAPH_SEPARATOR, '\n').rstrip('\0')
+        raw = str(c.selectedText()).replace(PARAGRAPH_SEPARATOR, '\n').rstrip('\0')
         m = pat.search(raw)
         if m is None:
             return False
@@ -506,7 +509,7 @@ class TextEdit(PlainTextEdit):
             if not found:
                 return False
         else:
-            raw = unicode_type(c.selectedText()).replace(PARAGRAPH_SEPARATOR, '\n').rstrip('\0')
+            raw = str(c.selectedText()).replace(PARAGRAPH_SEPARATOR, '\n').rstrip('\0')
             m = pat.search(raw)
             if m is None:
                 return False
@@ -539,7 +542,7 @@ class TextEdit(PlainTextEdit):
             return match_pos, match_word
 
         while True:
-            text = unicode_type(c.selectedText()).rstrip('\0')
+            text = str(c.selectedText()).rstrip('\0')
             idx, word = find_first_word(text)
             if idx == -1:
                 return False
@@ -576,7 +579,7 @@ class TextEdit(PlainTextEdit):
 
     def replace(self, pat, template, saved_match='gui'):
         c = self.textCursor()
-        raw = unicode_type(c.selectedText()).replace(PARAGRAPH_SEPARATOR, '\n').rstrip('\0')
+        raw = str(c.selectedText()).replace(PARAGRAPH_SEPARATOR, '\n').rstrip('\0')
         m = pat.fullmatch(raw)
         if m is None:
             # This can happen if either the user changed the selected text or
@@ -603,7 +606,7 @@ class TextEdit(PlainTextEdit):
             self.setTextCursor(c)
             return True
         base = r'''%%s\s*=\s*['"]{0,1}%s''' % regex.escape(anchor)
-        raw = unicode_type(self.toPlainText())
+        raw = str(self.toPlainText())
         m = regex.search(base % 'id', raw)
         if m is None:
             m = regex.search(base % 'name', raw)
@@ -682,7 +685,7 @@ class TextEdit(PlainTextEdit):
                     painter.setFont(f)
                     self.last_current_lnum = (top, bottom - top)
                 painter.drawText(0, top, self.line_number_area.width() - 5, self.fontMetrics().height(),
-                              Qt.AlignmentFlag.AlignRight, unicode_type(num + 1))
+                              Qt.AlignmentFlag.AlignRight, str(num + 1))
                 if current == num:
                     painter.restore()
             block = block.next()
@@ -713,7 +716,7 @@ class TextEdit(PlainTextEdit):
         c = self.textCursor()
         c.setPosition(block.position() + r.start)
         c.setPosition(c.position() + r.length, QTextCursor.MoveMode.KeepAnchor)
-        return unicode_type(c.selectedText())
+        return self.selected_text_from_cursor(c)
 
     def spellcheck_locale_for_cursor(self, c):
         with store_locale:
@@ -748,7 +751,7 @@ class TextEdit(PlainTextEdit):
         fmt_range = self.syntax_range_for_cursor(c)
         fmt = getattr(fmt_range, 'format', None)
         if fmt is not None:
-            tt = unicode_type(fmt.toolTip())
+            tt = str(fmt.toolTip())
             if tt:
                 QToolTip.setFont(self.tooltip_font)
                 QToolTip.setPalette(self.tooltip_palette)
@@ -764,6 +767,45 @@ class TextEdit(PlainTextEdit):
         if r is not None and r.format.property(LINK_PROPERTY):
             return self.text_for_range(c.block(), r)
 
+    def select_class_name_at_cursor(self, cursor):
+        valid = re.compile(r'[\w_0-9\-]+', flags=re.UNICODE)
+
+        def keep_going():
+            q = cursor.selectedText()
+            m = valid.match(q)
+            return m is not None and m.group() == q
+
+        def run_loop(forward=True):
+            cursor.setPosition(pos)
+            n, p = QTextCursor.MoveOperation.NextCharacter, QTextCursor.MoveOperation.PreviousCharacter
+            if not forward:
+                n, p = p, n
+            while True:
+                if not cursor.movePosition(n, QTextCursor.MoveMode.KeepAnchor):
+                    break
+                if not keep_going():
+                    cursor.movePosition(p, QTextCursor.MoveMode.KeepAnchor)
+                    break
+            ans = cursor.position()
+            cursor.setPosition(pos)
+            return ans
+
+        pos = cursor.position()
+        forwards_limit = run_loop()
+        backwards_limit = run_loop(forward=False)
+        cursor.setPosition(backwards_limit)
+        cursor.setPosition(forwards_limit, QTextCursor.MoveMode.KeepAnchor)
+        return self.selected_text_from_cursor(cursor)
+
+    def class_for_position(self, pos):
+        c = self.cursorForPosition(pos)
+        r = self.syntax_range_for_cursor(c)
+        if r is not None and r.format.property(CLASS_ATTRIBUTE_PROPERTY):
+            class_name = self.select_class_name_at_cursor(c)
+            if class_name:
+                tags = self.current_tag(for_position_sync=False, cursor=c)
+                return {'class': class_name, 'sourceline_address': tags}
+
     def mousePressEvent(self, ev):
         if self.completion_popup.isVisible() and not self.completion_popup.rect().contains(ev.pos()):
             # For some reason using eventFilter for this does not work, so we
@@ -775,6 +817,11 @@ class TextEdit(PlainTextEdit):
                 ev.accept()
                 self.link_clicked.emit(url)
                 return
+            class_data = self.class_for_position(ev.pos())
+            if class_data is not None:
+                ev.accept()
+                self.class_clicked.emit(class_data)
+                return
         return PlainTextEdit.mousePressEvent(self, ev)
 
     def get_range_inside_tag(self):
@@ -783,7 +830,7 @@ class TextEdit(PlainTextEdit):
         right = max(c.anchor(), c.position())
         # For speed we use QPlainTextEdit's toPlainText as we dont care about
         # spaces in this context
-        raw = unicode_type(QPlainTextEdit.toPlainText(self))
+        raw = str(QPlainTextEdit.toPlainText(self))
         # Make sure the left edge is not within a <>
         gtpos = raw.find('>', left)
         ltpos = raw.find('<', left)
@@ -829,7 +876,7 @@ class TextEdit(PlainTextEdit):
         c = self.textCursor()
         c.setPosition(left)
         c.setPosition(right, QTextCursor.MoveMode.KeepAnchor)
-        prev_text = unicode_type(c.selectedText()).rstrip('\0')
+        prev_text = str(c.selectedText()).rstrip('\0')
         c.insertText(prefix + prev_text + suffix)
         if prev_text:
             right = c.position()
@@ -861,7 +908,7 @@ version="1.1" width="100%%" height="100%%" viewBox="0 0 {w} {h}" preserveAspectR
 </svg></div>'''.format(w=width, h=height, a='xMidYMid meet' if preserve_aspect_ratio else 'none')
             else:
                 alt = _('Image')
-                template = '<img alt="{0}" src="%s" />'.format(alt)
+                template = '<img alt="{}" src="%s" />'.format(alt)
         text = template % href
         c.insertText(text)
         if self.syntax == 'html' and not fullpage:
@@ -884,6 +931,10 @@ version="1.1" width="100%%" height="100%%" viewBox="0 0 {w} {h}" preserveAspectR
         if hasattr(self.smarts, 'remove_tag'):
             self.smarts.remove_tag(self)
 
+    def split_tag(self):
+        if hasattr(self.smarts, 'split_tag'):
+            self.smarts.split_tag(self)
+
     def keyPressEvent(self, ev):
         if ev.key() == Qt.Key.Key_X and ev.modifiers() == Qt.KeyboardModifier.AltModifier:
             if self.replace_possible_unicode_sequence():
@@ -932,10 +983,10 @@ version="1.1" width="100%%" height="100%%" viewBox="0 0 {w} {h}" preserveAspectR
         c = self.textCursor()
         has_selection = c.hasSelection()
         if has_selection:
-            text = unicode_type(c.selectedText()).rstrip('\0')
+            text = str(c.selectedText()).rstrip('\0')
         else:
             c.setPosition(c.position() - min(c.positionInBlock(), 6), QTextCursor.MoveMode.KeepAnchor)
-            text = unicode_type(c.selectedText()).rstrip('\0')
+            text = str(c.selectedText()).rstrip('\0')
         m = re.search(r'[a-fA-F0-9]{2,6}$', text)
         if m is None:
             return False
@@ -962,8 +1013,12 @@ version="1.1" width="100%%" height="100%%" viewBox="0 0 {w} {h}" preserveAspectR
         if hasattr(self.smarts, 'rename_block_tag'):
             self.smarts.rename_block_tag(self, new_name)
 
-    def current_tag(self, for_position_sync=True):
-        return self.smarts.cursor_position_with_sourceline(self.textCursor(), for_position_sync=for_position_sync)
+    def current_tag(self, for_position_sync=True, cursor=None):
+        use_matched_tag = False
+        if cursor is None:
+            use_matched_tag = True
+            cursor = self.textCursor()
+        return self.smarts.cursor_position_with_sourceline(cursor, for_position_sync=for_position_sync, use_matched_tag=use_matched_tag)
 
     def goto_sourceline(self, sourceline, tags, attribute=None):
         return self.smarts.goto_sourceline(self, sourceline, tags, attribute=attribute)
@@ -977,7 +1032,7 @@ version="1.1" width="100%%" height="100%%" viewBox="0 0 {w} {h}" preserveAspectR
         from calibre.gui2.tweak_book.editor.smarts.css import find_rule
         block = None
         if self.syntax == 'css':
-            raw = unicode_type(self.toPlainText())
+            raw = str(self.toPlainText())
             line, col = find_rule(raw, rule_address)
             if line is not None:
                 block = self.document().findBlockByNumber(line - 1)
